# -*- coding: utf-8 -*-
'''
    :codeauthor: Nicole Thomas <nicole@saltstack.com>
    :codeauthor: Alexandru Bleotu <alexandru.bleotu@morganstanley.com>

    Tests for functions in salt.modules.vsphere
'''

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
from salt.ext.six import text_type
import salt.modules.vsphere as vsphere
from salt.exceptions import (
    CommandExecutionError,
    VMwareSaltError,
    ArgumentValueError,
    VMwareObjectRetrievalError
)

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON,
    call
)

# Import Third Party Libs
try:
    from pyVmomi import vim, vmodl  # pylint: disable=unused-import
    HAS_PYVMOMI = True
except ImportError:
    HAS_PYVMOMI = False

# Globals
HOST = '1.2.3.4'
USER = 'root'
PASSWORD = 'SuperSecret!'
ERROR = 'Some Testing Error Message'


@skipIf(NO_MOCK, NO_MOCK_REASON)
class VsphereTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Unit TestCase for the salt.modules.vsphere module.
    '''
    def setup_loader_modules(self):
        return {vsphere: {'__virtual__': MagicMock(return_value='vsphere')}}

    # Tests for get_coredump_network_config function

    def test_get_coredump_network_config_esxi_hosts_not_list(self):
        '''
        Tests CommandExecutionError is raised when esxi_hosts is provided,
        but is not a list.
        '''
        self.assertRaises(CommandExecutionError,
                          vsphere.get_coredump_network_config,
                          HOST, USER, PASSWORD, esxi_hosts='foo')

    def test_get_coredump_network_config_host_list_bad_retcode(self):
        '''
        Tests error message returned with list of esxi_hosts.
        '''
        with patch('salt.utils.vmware.esxcli', MagicMock(return_value={'retcode': 1, 'stdout': ERROR})):
            host_1 = 'host_1.foo.com'
            self.assertEqual({host_1: {'Error': ERROR}},
                             vsphere.get_coredump_network_config(HOST, USER, PASSWORD, esxi_hosts=[host_1]))

    def test_get_coredump_network_config_host_list_success(self):
        '''
        Tests successful function return when an esxi_host is provided.
        '''
        with patch('salt.utils.vmware.esxcli', MagicMock(return_value={'retcode': 0, 'stdout': ''})):
            with patch('salt.modules.vsphere._format_coredump_stdout', MagicMock(return_value={})):
                host_1 = 'host_1.foo.com'
                self.assertEqual({host_1: {'Coredump Config': {}}},
                                 vsphere.get_coredump_network_config(HOST, USER, PASSWORD, esxi_hosts=[host_1]))

    def test_get_coredump_network_config_bad_retcode(self):
        '''
        Tests error message given for a single ESXi host.
        '''
        with patch('salt.utils.vmware.esxcli', MagicMock(return_value={'retcode': 1, 'stdout': ERROR})):
            self.assertEqual({HOST: {'Error': ERROR}},
                             vsphere.get_coredump_network_config(HOST, USER, PASSWORD))

    def test_get_coredump_network_config_success(self):
        '''
        Tests successful function return for a single ESXi host.
        '''
        with patch('salt.utils.vmware.esxcli', MagicMock(return_value={'retcode': 0, 'stdout': ''})):
            with patch('salt.modules.vsphere._format_coredump_stdout', MagicMock(return_value={})):
                self.assertEqual({HOST: {'Coredump Config': {}}},
                                 vsphere.get_coredump_network_config(HOST, USER, PASSWORD))

    # Tests for coredump_network_enable function

    def test_coredump_network_enable_esxi_hosts_not_list(self):
        '''
        Tests CommandExecutionError is raised when esxi_hosts is provided,
        but is not a list.
        '''
        self.assertRaises(CommandExecutionError,
                          vsphere.coredump_network_enable,
                          HOST, USER, PASSWORD, True, esxi_hosts='foo')

    def test_coredump_network_enable_host_list_bad_retcode(self):
        '''
        Tests error message returned with list of esxi_hosts.
        '''
        with patch('salt.utils.vmware.esxcli', MagicMock(return_value={'retcode': 1, 'stdout': ERROR})):
            host_1 = 'host_1.foo.com'
            self.assertEqual({host_1: {'Error': ERROR}},
                             vsphere.coredump_network_enable(HOST, USER, PASSWORD, True, esxi_hosts=[host_1]))

    def test_coredump_network_enable_host_list_success(self):
        '''
        Tests successful function return when an esxi_host is provided.
        '''
        with patch('salt.utils.vmware.esxcli', MagicMock(return_value={'retcode': 0, 'stdout': ''})):
            with patch('salt.modules.vsphere._format_coredump_stdout', MagicMock(return_value={})):
                enabled = True
                host_1 = 'host_1.foo.com'
                self.assertEqual({host_1: {'Coredump Enabled': enabled}},
                                 vsphere.coredump_network_enable(HOST, USER, PASSWORD, enabled, esxi_hosts=[host_1]))

    def test_coredump_network_enable_bad_retcode(self):
        '''
        Tests error message given for a single ESXi host.
        '''
        with patch('salt.utils.vmware.esxcli', MagicMock(return_value={'retcode': 1, 'stdout': ERROR})):
            self.assertEqual({HOST: {'Error': ERROR}},
                             vsphere.coredump_network_enable(HOST, USER, PASSWORD, True))

    def test_coredump_network_enable_success(self):
        '''
        Tests successful function return for a single ESXi host.
        '''
        with patch('salt.utils.vmware.esxcli', MagicMock(return_value={'retcode': 0, 'stdout': ''})):
            with patch('salt.modules.vsphere._format_coredump_stdout', MagicMock(return_value={})):
                enabled = True
                self.assertEqual({HOST: {'Coredump Enabled': enabled}},
                                 vsphere.coredump_network_enable(HOST, USER, PASSWORD, enabled))

    # Tests for set_coredump_network_config function

    def test_set_coredump_network_config_esxi_hosts_not_list(self):
        '''
        Tests CommandExecutionError is raised when esxi_hosts is provided,
        but is not a list.
        '''
        self.assertRaises(CommandExecutionError,
                          vsphere.set_coredump_network_config,
                          HOST, USER, PASSWORD, 'loghost', 'foo', esxi_hosts='bar')

    def test_set_coredump_network_config_host_list_bad_retcode(self):
        '''
        Tests error message returned with list of esxi_hosts.
        '''
        with patch('salt.utils.vmware.esxcli', MagicMock(return_value={'retcode': 1})):
            host_1 = 'host_1.foo.com'
            self.assertEqual({host_1: {'retcode': 1, 'success': False}},
                             vsphere.set_coredump_network_config(HOST,
                                                                 USER,
                                                                 PASSWORD,
                                                                 'dump-ip.test.com',
                                                                 esxi_hosts=[host_1]))

    def test_set_coredump_network_config_host_list_success(self):
        '''
        Tests successful function return when an esxi_host is provided.
        '''
        with patch('salt.utils.vmware.esxcli', MagicMock(return_value={'retcode': 0})):
            host_1 = 'host_1.foo.com'
            self.assertEqual({host_1: {'retcode': 0, 'success': True}},
                             vsphere.set_coredump_network_config(HOST,
                                                                 USER,
                                                                 PASSWORD,
                                                                 'dump-ip.test.com',
                                                                 esxi_hosts=[host_1]))

    def test_set_coredump_network_config_bad_retcode(self):
        '''
        Tests error message given for a single ESXi host.
        '''
        with patch('salt.utils.vmware.esxcli', MagicMock(return_value={'retcode': 1})):
            self.assertEqual({HOST: {'retcode': 1, 'success': False}},
                             vsphere.set_coredump_network_config(HOST,
                                                                 USER,
                                                                 PASSWORD,
                                                                 'dump-ip.test.com'))

    def test_set_coredump_network_config_success(self):
        '''
        Tests successful function return for a single ESXi host.
        '''
        with patch('salt.utils.vmware.esxcli', MagicMock(return_value={'retcode': 0})):
            self.assertEqual({HOST: {'retcode': 0, 'success': True}},
                             vsphere.set_coredump_network_config(HOST,
                                                                 USER,
                                                                 PASSWORD,
                                                                 'dump-ip.test.com'))

    # Tests for get_firewall_status function

    def test_get_firewall_status_esxi_hosts_not_list(self):
        '''
        Tests CommandExecutionError is raised when esxi_hosts is provided,
        but is not a list.
        '''
        self.assertRaises(CommandExecutionError,
                          vsphere.get_firewall_status,
                          HOST, USER, PASSWORD, esxi_hosts='foo')

    def test_get_firewall_status_host_list_bad_retcode(self):
        '''
        Tests error message returned with list of esxi_hosts.
        '''
        with patch('salt.utils.vmware.esxcli', MagicMock(return_value={'retcode': 1, 'stdout': ERROR})):
            host_1 = 'host_1.foo.com'
            self.assertEqual({host_1: {'success': False, 'Error': ERROR, 'rulesets': None}},
                             vsphere.get_firewall_status(HOST, USER, PASSWORD, esxi_hosts=[host_1]))

    def test_get_firewall_status_host_list_success(self):
        '''
        Tests successful function return when an esxi_host is provided.
        '''
        with patch('salt.utils.vmware.esxcli', MagicMock(return_value={'retcode': 0, 'stdout': ''})):
            host_1 = 'host_1.foo.com'
            self.assertEqual({host_1: {'rulesets': {}, 'success': True}},
                             vsphere.get_firewall_status(HOST, USER, PASSWORD, esxi_hosts=[host_1]))

    def test_get_firewall_status_bad_retcode(self):
        '''
        Tests error message given for a single ESXi host.
        '''
        with patch('salt.utils.vmware.esxcli', MagicMock(return_value={'retcode': 1, 'stdout': ERROR})):
            self.assertEqual({HOST: {'success': False, 'Error': ERROR, 'rulesets': None}},
                             vsphere.get_firewall_status(HOST, USER, PASSWORD))

    def test_get_firewall_status_success(self):
        '''
        Tests successful function return for a single ESXi host.
        '''
        with patch('salt.utils.vmware.esxcli', MagicMock(return_value={'retcode': 0, 'stdout': ''})):
            self.assertEqual({HOST: {'rulesets': {}, 'success': True}},
                             vsphere.get_firewall_status(HOST, USER, PASSWORD))

    # Tests for enable_firewall_ruleset function

    def test_enable_firewall_ruleset_esxi_hosts_not_list(self):
        '''
        Tests CommandExecutionError is raised when esxi_hosts is provided,
        but is not a list.
        '''
        self.assertRaises(CommandExecutionError,
                          vsphere.enable_firewall_ruleset,
                          HOST, USER, PASSWORD, 'foo', 'bar', esxi_hosts='baz')

    # Tests for syslog_service_reload function

    def test_syslog_service_reload_esxi_hosts_not_list(self):
        '''
        Tests CommandExecutionError is raised when esxi_hosts is provided,
        but is not a list.
        '''
        self.assertRaises(CommandExecutionError,
                          vsphere.syslog_service_reload,
                          HOST, USER, PASSWORD, esxi_hosts='foo')

    # Tests for set_syslog_config function.
    # These tests only test the firewall=True and syslog_config == 'loghost' if block.
    # The rest of the function is tested in the _set_syslog_config_helper tests below.

    def test_set_syslog_config_esxi_hosts_not_list(self):
        '''
        Tests CommandExecutionError is raised when esxi_hosts is provided,
        but is not a list, but we don't enter the 'loghost'/firewall loop.
        '''
        self.assertRaises(CommandExecutionError,
                          vsphere.set_syslog_config,
                          HOST, USER, PASSWORD, 'foo', 'bar', esxi_hosts='baz')

    def test_set_syslog_config_esxi_hosts_not_list_firewall(self):
        '''
        Tests CommandExecutionError is raised when esxi_hosts is provided,
        but is not a list, and we enter the 'loghost'/firewall loop.
        '''
        self.assertRaises(CommandExecutionError,
                          vsphere.set_syslog_config,
                          HOST, USER, PASSWORD, 'loghost', 'foo', firewall=True, esxi_hosts='bar')

    def test_set_syslog_config_host_list_firewall_bad_retcode(self):
        '''
        Tests error message returned with list of esxi_hosts with 'loghost' as syslog_config.
        '''
        with patch('salt.modules.vsphere.enable_firewall_ruleset',
                   MagicMock(return_value={'host_1.foo.com': {'retcode': 1, 'stdout': ERROR}})):
            with patch('salt.modules.vsphere._set_syslog_config_helper',
                       MagicMock(return_value={})):
                host_1 = 'host_1.foo.com'
                self.assertEqual({host_1: {'enable_firewall': {'message': ERROR, 'success': False}}},
                                 vsphere.set_syslog_config(HOST,
                                                           USER,
                                                           PASSWORD,
                                                           'loghost',
                                                           'foo',
                                                           firewall=True,
                                                           esxi_hosts=[host_1]))

    def test_set_syslog_config_host_list_firewall_success(self):
        '''
        Tests successful function return with list of esxi_hosts with 'loghost' as syslog_config.
        '''
        with patch('salt.modules.vsphere.enable_firewall_ruleset',
                   MagicMock(return_value={'host_1.foo.com': {'retcode': 0}})):
            with patch('salt.modules.vsphere._set_syslog_config_helper',
                       MagicMock(return_value={})):
                host_1 = 'host_1.foo.com'
                self.assertEqual({host_1: {'enable_firewall': {'success': True}}},
                                 vsphere.set_syslog_config(HOST,
                                                           USER,
                                                           PASSWORD,
                                                           'loghost',
                                                           'foo',
                                                           firewall=True,
                                                           esxi_hosts=[host_1]))

    def test_set_syslog_config_firewall_bad_retcode(self):
        '''
        Tests error message given for a single ESXi host with 'loghost' as syslog_config.
        '''
        with patch('salt.modules.vsphere.enable_firewall_ruleset',
                   MagicMock(return_value={HOST: {'retcode': 1, 'stdout': ERROR}})):
            with patch('salt.modules.vsphere._set_syslog_config_helper',
                       MagicMock(return_value={})):
                self.assertEqual({HOST: {'enable_firewall': {'message': ERROR, 'success': False}}},
                                 vsphere.set_syslog_config(HOST,
                                                           USER,
                                                           PASSWORD,
                                                           'loghost',
                                                           'foo',
                                                           firewall=True))

    def test_set_syslog_config_firewall_success(self):
        '''
        Tests successful function return for a single ESXi host with 'loghost' as syslog_config.
        '''
        with patch('salt.modules.vsphere.enable_firewall_ruleset',
                   MagicMock(return_value={HOST: {'retcode': 0}})):
            with patch('salt.modules.vsphere._set_syslog_config_helper',
                       MagicMock(return_value={})):
                self.assertEqual({HOST: {'enable_firewall': {'success': True}}},
                                 vsphere.set_syslog_config(HOST,
                                                           USER,
                                                           PASSWORD,
                                                           'loghost',
                                                           'foo',
                                                           firewall=True))

    # Tests for get_syslog_config function

    def test_get_syslog_config_esxi_hosts_not_list(self):
        '''
        Tests CommandExecutionError is raised when esxi_hosts is provided,
        but is not a list.
        '''
        self.assertRaises(CommandExecutionError,
                          vsphere.get_syslog_config,
                          HOST, USER, PASSWORD, esxi_hosts='foo')

    def test_get_syslog_config_host_list_bad_retcode(self):
        '''
        Tests error message returned with list of esxi_hosts.
        '''
        with patch('salt.utils.vmware.esxcli', MagicMock(return_value={'retcode': 1, 'stdout': ERROR})):
            host_1 = 'host_1.foo.com'
            self.assertEqual({host_1: {'message': ERROR, 'success': False}},
                             vsphere.get_syslog_config(HOST, USER, PASSWORD, esxi_hosts=[host_1]))

    def test_get_syslog_config_host_list_success(self):
        '''
        Tests successful function return when an esxi_host is provided.
        '''
        with patch('salt.utils.vmware.esxcli', MagicMock(return_value={'retcode': 0, 'stdout': ''})):
            host_1 = 'host_1.foo.com'
            self.assertEqual({host_1: {'success': True}},
                             vsphere.get_syslog_config(HOST, USER, PASSWORD, esxi_hosts=[host_1]))

    def test_get_syslog_config_bad_retcode(self):
        '''
        Tests error message given for a single ESXi host.
        '''
        with patch('salt.utils.vmware.esxcli', MagicMock(return_value={'retcode': 1, 'stdout': ERROR})):
            self.assertEqual({HOST: {'message': ERROR, 'success': False}},
                             vsphere.get_syslog_config(HOST, USER, PASSWORD))

    def test_get_syslog_config_success(self):
        '''
        Tests successful function return for a single ESXi host.
        '''
        with patch('salt.utils.vmware.esxcli', MagicMock(return_value={'retcode': 0, 'stdout': ''})):
            self.assertEqual({HOST: {'success': True}},
                             vsphere.get_syslog_config(HOST, USER, PASSWORD))

    # Tests for reset_syslog_config function

    def test_reset_syslog_config_no_syslog_config(self):
        '''
        Tests CommandExecutionError is raised when a syslog_config parameter is missing.
        '''
        self.assertRaises(CommandExecutionError,
                          vsphere.reset_syslog_config,
                          HOST, USER, PASSWORD)

    def test_reset_syslog_config_esxi_hosts_not_list(self):
        '''
        Tests CommandExecutionError is raised when esxi_hosts is provided,
        but is not a list.
        '''
        self.assertRaises(CommandExecutionError,
                          vsphere.reset_syslog_config,
                          HOST, USER, PASSWORD, syslog_config='test', esxi_hosts='foo')

    def test_reset_syslog_config_invalid_config_param(self):
        '''
        Tests error message returned when an invalid syslog_config parameter is provided.
        '''
        with patch('salt.utils.vmware.esxcli', MagicMock(return_value={})):
            error = 'Invalid syslog configuration parameter'
            self.assertEqual({HOST: {'success': False, 'test': {'message': error, 'success': False}}},
                             vsphere.reset_syslog_config(HOST, USER, PASSWORD,
                                                         syslog_config='test'))

    def test_reset_syslog_config_host_list_bad_retcode(self):
        '''
        Tests error message returned with list of esxi_hosts.
        '''
        with patch('salt.utils.vmware.esxcli', MagicMock(return_value={'retcode': 1, 'stdout': ERROR})):
            host_1 = 'host_1.foo.com'
            self.assertEqual({host_1: {'success': False, 'logdir': {'message': ERROR, 'success': False}}},
                             vsphere.reset_syslog_config(HOST, USER, PASSWORD,
                                                         syslog_config='logdir',
                                                         esxi_hosts=[host_1]))

    def test_reset_syslog_config_host_list_success(self):
        '''
        Tests successful function return when an esxi_host is provided.
        '''
        with patch('salt.utils.vmware.esxcli', MagicMock(return_value={'retcode': 0, 'stdout': ''})):
            host_1 = 'host_1.foo.com'
            self.assertEqual({host_1: {'success': True, 'loghost': {'success': True}}},
                             vsphere.reset_syslog_config(HOST, USER, PASSWORD,
                                                         syslog_config='loghost',
                                                         esxi_hosts=[host_1]))

    def test_reset_syslog_config_bad_retcode(self):
        '''
        Tests error message given for a single ESXi host.
        '''
        with patch('salt.utils.vmware.esxcli', MagicMock(return_value={'retcode': 1, 'stdout': ERROR})):
            self.assertEqual({HOST: {'success': False, 'logdir-unique': {'message': ERROR, 'success': False}}},
                             vsphere.reset_syslog_config(HOST, USER, PASSWORD,
                                                         syslog_config='logdir-unique'))

    def test_reset_syslog_config_success(self):
        '''
        Tests successful function return for a single ESXi host.
        '''
        with patch('salt.utils.vmware.esxcli', MagicMock(return_value={'retcode': 0, 'stdout': ''})):
            self.assertEqual({HOST: {'success': True, 'default-rotate': {'success': True}}},
                             vsphere.reset_syslog_config(HOST, USER, PASSWORD,
                                                         syslog_config='default-rotate'))

    def test_reset_syslog_config_success_multiple_configs(self):
        '''
        Tests successful function return for a single ESXi host when passing in multiple syslog_config values.
        '''
        with patch('salt.utils.vmware.esxcli', MagicMock(return_value={'retcode': 0, 'stdout': ''})):
            self.assertEqual({HOST: {'success': True,
                                     'default-size': {'success': True},
                                     'default-timeout': {'success': True}}},
                             vsphere.reset_syslog_config(HOST, USER, PASSWORD,
                                                         syslog_config='default-size,default-timeout'))

    def test_reset_syslog_config_success_all_configs(self):
        '''
        Tests successful function return for a single ESXi host when passing in multiple syslog_config values.
        '''
        with patch('salt.utils.vmware.esxcli', MagicMock(return_value={'retcode': 0, 'stdout': ''})):
            self.assertEqual({HOST: {'success': True,
                                     'logdir': {'success': True},
                                     'loghost': {'success': True},
                                     'default-rotate': {'success': True},
                                     'default-size': {'success': True},
                                     'default-timeout': {'success': True},
                                     'logdir-unique': {'success': True}}},
                             vsphere.reset_syslog_config(HOST, USER, PASSWORD,
                                                         syslog_config='all'))

    # Tests for _reset_syslog_config_params function

    def test_reset_syslog_config_params_no_valid_reset(self):
        '''
        Tests function returns False when an invalid syslog config is passed.
        '''
        valid_resets = ['hello', 'world']
        config = 'foo'
        ret = {'success': False, config: {'success': False, 'message': 'Invalid syslog configuration parameter'}}
        self.assertEqual(ret, vsphere._reset_syslog_config_params(HOST, USER, PASSWORD,
                                                                  'cmd', config, valid_resets))

    def test_reset_syslog_config_params_error(self):
        '''
        Tests function returns False when the esxxli function returns an unsuccessful retcode.
        '''
        with patch('salt.utils.vmware.esxcli', MagicMock(return_value={'retcode': 1, 'stdout': ERROR})):
            valid_resets = ['hello', 'world']
            error_dict = {'success': False, 'message': ERROR}
            ret = {'success': False, 'hello': error_dict, 'world': error_dict}
            self.assertDictEqual(ret, vsphere._reset_syslog_config_params(HOST, USER, PASSWORD,
                                                                          'cmd', valid_resets, valid_resets))

    def test_reset_syslog_config_params_success(self):
        '''
        Tests function returns True when the esxxli function returns a successful retcode.
        '''
        with patch('salt.utils.vmware.esxcli', MagicMock(return_value={'retcode': 0})):
            valid_resets = ['hello', 'world']
            ret = {'success': True, 'hello': {'success': True}, 'world': {'success': True}}
            self.assertDictEqual(ret, vsphere._reset_syslog_config_params(HOST, USER, PASSWORD,
                                                                          'cmd', valid_resets, valid_resets))

    # Tests for _set_syslog_config_helper function

    def test_set_syslog_config_helper_no_valid_reset(self):
        '''
        Tests function returns False when an invalid syslog config is passed.
        '''
        config = 'foo'
        ret = {'success': False, 'message': '\'{0}\' is not a valid config variable.'.format(config)}
        self.assertEqual(ret, vsphere._set_syslog_config_helper(HOST, USER, PASSWORD, config, 'bar'))

    def test_set_syslog_config_helper_bad_retcode(self):
        '''
        Tests function returns False when the esxcli function returns an unsuccessful retcode.
        '''
        with patch('salt.utils.vmware.esxcli', MagicMock(return_value={'retcode': 1, 'stdout': ERROR})):
            config = 'default-rotate'
            self.assertEqual({config: {'success': False, 'message': ERROR}},
                             vsphere._set_syslog_config_helper(HOST, USER, PASSWORD, config, 'foo'))

    def test_set_syslog_config_helper_success(self):
        '''
        Tests successful function return.
        '''
        with patch('salt.utils.vmware.esxcli', MagicMock(return_value={'retcode': 0})):
            config = 'logdir'
            self.assertEqual({config: {'success': True}},
                             vsphere._set_syslog_config_helper(HOST, USER, PASSWORD, config, 'foo'))


@skipIf(NO_MOCK, NO_MOCK_REASON)
class GetProxyTypeTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Tests for salt.modules.vsphere.get_proxy_type
    '''
    def setup_loader_modules(self):
        return {vsphere: {'__virtual__': MagicMock(return_value='vsphere')}}

    def test_output(self):
        with patch.dict(vsphere.__pillar__,
                        {'proxy': {'proxytype': 'fake_proxy_type'}}):
            ret = vsphere.get_proxy_type()
        self.assertEqual('fake_proxy_type', ret)


@skipIf(NO_MOCK, NO_MOCK_REASON)
class SupportsProxiesTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Tests for salt.modules.vsphere.supports_proxies decorator
    '''
    def setup_loader_modules(self):
        return {vsphere: {'__virtual__': MagicMock(return_value='vsphere')}}

    def test_supported_proxy(self):
        @vsphere.supports_proxies('supported')
        def mock_function():
            return 'fake_function'

        with patch('salt.modules.vsphere.get_proxy_type',
                   MagicMock(return_value='supported')):
            ret = mock_function()
        self.assertEqual('fake_function', ret)

    def test_unsupported_proxy(self):
        @vsphere.supports_proxies('supported')
        def mock_function():
            return 'fake_function'

        with patch('salt.modules.vsphere.get_proxy_type',
                   MagicMock(return_value='unsupported')):
            with self.assertRaises(CommandExecutionError) as excinfo:
                mock_function()
        self.assertEqual('\'unsupported\' proxy is not supported by '
                         'function mock_function',
                         excinfo.exception.strerror)


@skipIf(NO_MOCK, NO_MOCK_REASON)
class _GetProxyConnectionDetailsTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Tests for salt.modules.vsphere._get_proxy_connection_details
    '''
    def setup_loader_modules(self):
        return {vsphere: {'__virtual__': MagicMock(return_value='vsphere')}}

    def setUp(self):
        self.esxi_host_details = {'host': 'fake_host',
                                  'username': 'fake_username',
                                  'password': 'fake_password',
                                  'protocol': 'fake_protocol',
                                  'port': 'fake_port',
                                  'mechanism': 'fake_mechanism',
                                  'principal': 'fake_principal',
                                  'domain': 'fake_domain'}
        self.esxi_vcenter_details = {'vcenter': 'fake_vcenter',
                                     'username': 'fake_username',
                                     'password': 'fake_password',
                                     'protocol': 'fake_protocol',
                                     'port': 'fake_port',
                                     'mechanism': 'fake_mechanism',
                                     'principal': 'fake_principal',
                                     'domain': 'fake_domain'}
        self.esxdatacenter_details = {'vcenter': 'fake_vcenter',
                                      'datacenter': 'fake_dc',
                                      'username': 'fake_username',
                                      'password': 'fake_password',
                                      'protocol': 'fake_protocol',
                                      'port': 'fake_port',
                                      'mechanism': 'fake_mechanism',
                                      'principal': 'fake_principal',
                                      'domain': 'fake_domain'}
        self.esxcluster_details = {'vcenter': 'fake_vcenter',
                                   'datacenter': 'fake_dc',
                                   'cluster': 'fake_cluster',
                                   'username': 'fake_username',
                                   'password': 'fake_password',
                                   'protocol': 'fake_protocol',
                                   'port': 'fake_port',
                                   'mechanism': 'fake_mechanism',
                                   'principal': 'fake_principal',
                                   'domain': 'fake_domain'}
        self.vcenter_details = {'vcenter': 'fake_vcenter',
                                'username': 'fake_username',
                                'password': 'fake_password',
                                'protocol': 'fake_protocol',
                                'port': 'fake_port',
                                'mechanism': 'fake_mechanism',
                                'principal': 'fake_principal',
                                'domain': 'fake_domain'}

    def tearDown(self):
        for attrname in ('esxi_host_details', 'esxi_vcenter_details',
                         'esxdatacenter_details', 'esxcluster_details'):
            try:
                delattr(self, attrname)
            except AttributeError:
                continue

    def test_esxi_proxy_host_details(self):
        with patch('salt.modules.vsphere.get_proxy_type',
                   MagicMock(return_value='esxi')):
            with patch.dict(vsphere.__salt__,
                            {'esxi.get_details':
                             MagicMock(return_value=self.esxi_host_details)}):
                ret = vsphere._get_proxy_connection_details()
        self.assertEqual(('fake_host', 'fake_username', 'fake_password',
                          'fake_protocol', 'fake_port', 'fake_mechanism',
                          'fake_principal', 'fake_domain'), ret)

    def test_esxdatacenter_proxy_details(self):
        with patch('salt.modules.vsphere.get_proxy_type',
                   MagicMock(return_value='esxdatacenter')):
            with patch.dict(vsphere.__salt__,
                            {'esxdatacenter.get_details': MagicMock(
                             return_value=self.esxdatacenter_details)}):
                ret = vsphere._get_proxy_connection_details()
        self.assertEqual(('fake_vcenter', 'fake_username', 'fake_password',
                          'fake_protocol', 'fake_port', 'fake_mechanism',
                          'fake_principal', 'fake_domain'), ret)

    def test_esxcluster_proxy_details(self):
        with patch('salt.modules.vsphere.get_proxy_type',
                   MagicMock(return_value='esxcluster')):
            with patch.dict(vsphere.__salt__,
                            {'esxcluster.get_details': MagicMock(
                             return_value=self.esxcluster_details)}):
                ret = vsphere._get_proxy_connection_details()
        self.assertEqual(('fake_vcenter', 'fake_username', 'fake_password',
                          'fake_protocol', 'fake_port', 'fake_mechanism',
                          'fake_principal', 'fake_domain'), ret)

    def test_esxi_proxy_vcenter_details(self):
        with patch('salt.modules.vsphere.get_proxy_type',
                   MagicMock(return_value='esxi')):
            with patch.dict(vsphere.__salt__,
                            {'esxi.get_details':
                             MagicMock(
                                 return_value=self.esxi_vcenter_details)}):
                ret = vsphere._get_proxy_connection_details()
        self.assertEqual(('fake_vcenter', 'fake_username', 'fake_password',
                          'fake_protocol', 'fake_port', 'fake_mechanism',
                          'fake_principal', 'fake_domain'), ret)

    def test_vcenter_proxy_details(self):
        with patch('salt.modules.vsphere.get_proxy_type',
                   MagicMock(return_value='vcenter')):
            with patch.dict(vsphere.__salt__,
                            {'vcenter.get_details': MagicMock(
                             return_value=self.vcenter_details)}):
                ret = vsphere._get_proxy_connection_details()
        self.assertEqual(('fake_vcenter', 'fake_username', 'fake_password',
                          'fake_protocol', 'fake_port', 'fake_mechanism',
                          'fake_principal', 'fake_domain'), ret)

    def test_unsupported_proxy_details(self):
        with patch('salt.modules.vsphere.get_proxy_type',
                   MagicMock(return_value='unsupported')):
            with self.assertRaises(CommandExecutionError) as excinfo:
                ret = vsphere._get_proxy_connection_details()
        self.assertEqual('\'unsupported\' proxy is not supported',
                         excinfo.exception.strerror)


@skipIf(NO_MOCK, NO_MOCK_REASON)
class GetsServiceInstanceViaProxyTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Tests for salt.modules.vsphere.gets_service_instance_via_proxy
    decorator
    '''
    def setup_loader_modules(self):
        patcher = patch('salt.utils.vmware.get_service_instance', MagicMock())
        patcher.start()
        self.addCleanup(patcher.stop)
        patcher = patch('salt.utils.vmware.disconnect', MagicMock())
        patcher.start()
        self.addCleanup(patcher.stop)
        return {
            vsphere: {
                '__virtual__': MagicMock(return_value='vsphere'),
                '_get_proxy_connection_details': MagicMock(),
            }
        }

    def setUp(self):
        self.mock_si = MagicMock()
        self.mock_details1 = MagicMock()
        self.mock_details2 = MagicMock()

    def tearDown(self):
        for attrname in ('mock_si', 'mock_details1', 'mock_details2'):
            try:
                delattr(self, attrname)
            except AttributeError:
                continue

    def test_no_service_instance_or_kwargs_parameters(self):
        @vsphere.gets_service_instance_via_proxy
        def mock_function():
            return 'fake_function'

        with self.assertRaises(CommandExecutionError) as excinfo:
            mock_function()
        self.assertEqual('Function mock_function must have either a '
                         '\'service_instance\', or a \'**kwargs\' type '
                         'parameter', excinfo.exception.strerror)

    def test___get_proxy_connection_details_call(self):
        mock__get_proxy_connection_details = MagicMock()

        @vsphere.gets_service_instance_via_proxy
        def mock_function(service_instance=None):
            return service_instance

        with patch('salt.modules.vsphere._get_proxy_connection_details',
                   mock__get_proxy_connection_details):
            mock_function()
        mock__get_proxy_connection_details.assert_called_once_with()

    def test_service_instance_named_parameter_no_value(self):
        mock_get_service_instance = MagicMock(return_value=self.mock_si)
        mock_disconnect = MagicMock()

        @vsphere.gets_service_instance_via_proxy
        def mock_function(service_instance=None):
            return service_instance

        with patch('salt.modules.vsphere._get_proxy_connection_details',
                   MagicMock(return_value=(self.mock_details1,
                                           self.mock_details2))):
            with patch('salt.utils.vmware.get_service_instance',
                       mock_get_service_instance):
                with patch('salt.utils.vmware.disconnect', mock_disconnect):
                    ret = mock_function()
        mock_get_service_instance.assert_called_once_with(self.mock_details1,
                                                          self.mock_details2)
        mock_disconnect.assert_called_once_with(self.mock_si)
        self.assertEqual(ret, self.mock_si)

    def test_service_instance_kwargs_parameter_no_value(self):
        mock_get_service_instance = MagicMock(return_value=self.mock_si)
        mock_disconnect = MagicMock()

        @vsphere.gets_service_instance_via_proxy
        def mock_function(**kwargs):
            return kwargs['service_instance']

        with patch('salt.modules.vsphere._get_proxy_connection_details',
                   MagicMock(return_value=(self.mock_details1,
                                           self.mock_details2))):
            with patch('salt.utils.vmware.get_service_instance',
                       mock_get_service_instance):
                with patch('salt.utils.vmware.disconnect', mock_disconnect):
                    ret = mock_function()
        mock_get_service_instance.assert_called_once_with(self.mock_details1,
                                                          self.mock_details2)
        mock_disconnect.assert_called_once_with(self.mock_si)
        self.assertEqual(ret, self.mock_si)

    def test_service_instance_positional_parameter_no_default_value(self):
        mock_get_service_instance = MagicMock()
        mock_disconnect = MagicMock()

        @vsphere.gets_service_instance_via_proxy
        def mock_function(service_instance):
            return service_instance

        with patch('salt.modules.vsphere._get_proxy_connection_details',
                   MagicMock(return_value=(self.mock_details1,
                                           self.mock_details2))):
            with patch('salt.utils.vmware.get_service_instance',
                       mock_get_service_instance):
                with patch('salt.utils.vmware.disconnect', mock_disconnect):
                    ret = mock_function(self.mock_si)
        self.assertEqual(mock_get_service_instance.call_count, 0)
        self.assertEqual(mock_disconnect.call_count, 0)
        self.assertEqual(ret, self.mock_si)

    def test_service_instance_positional_parameter_with_default_value(self):
        mock_get_service_instance = MagicMock()
        mock_disconnect = MagicMock()

        @vsphere.gets_service_instance_via_proxy
        def mock_function(service_instance=None):
            return service_instance

        with patch('salt.modules.vsphere._get_proxy_connection_details',
                   MagicMock(return_value=(self.mock_details1,
                                           self.mock_details2))):
            with patch('salt.utils.vmware.get_service_instance',
                       mock_get_service_instance):
                with patch('salt.utils.vmware.disconnect', mock_disconnect):
                    ret = mock_function(self.mock_si)
        self.assertEqual(mock_get_service_instance.call_count, 0)
        self.assertEqual(mock_disconnect.call_count, 0)
        self.assertEqual(ret, self.mock_si)

    def test_service_instance_named_parameter_with_default_value(self):
        mock_get_service_instance = MagicMock()
        mock_disconnect = MagicMock()

        @vsphere.gets_service_instance_via_proxy
        def mock_function(service_instance=None):
            return service_instance

        with patch('salt.modules.vsphere._get_proxy_connection_details',
                   MagicMock(return_value=(self.mock_details1,
                                           self.mock_details2))):
            with patch('salt.utils.vmware.get_service_instance',
                       mock_get_service_instance):
                with patch('salt.utils.vmware.disconnect', mock_disconnect):
                    ret = mock_function(service_instance=self.mock_si)
        self.assertEqual(mock_get_service_instance.call_count, 0)
        self.assertEqual(mock_disconnect.call_count, 0)
        self.assertEqual(ret, self.mock_si)

    def test_service_instance_kwargs_parameter_passthrough(self):
        mock_get_service_instance = MagicMock()
        mock_disconnect = MagicMock()

        @vsphere.gets_service_instance_via_proxy
        def mock_function(**kwargs):
            return kwargs['service_instance']

        with patch('salt.modules.vsphere._get_proxy_connection_details',
                   MagicMock(return_value=(self.mock_details1,
                                           self.mock_details2))):
            with patch('salt.utils.vmware.get_service_instance',
                       mock_get_service_instance):
                with patch('salt.utils.vmware.disconnect', mock_disconnect):
                    ret = mock_function(service_instance=self.mock_si)
        self.assertEqual(mock_get_service_instance.call_count, 0)
        self.assertEqual(mock_disconnect.call_count, 0)
        self.assertEqual(ret, self.mock_si)


@skipIf(NO_MOCK, NO_MOCK_REASON)
class GetServiceInstanceViaProxyTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Tests for salt.modules.vsphere.get_service_instance_via_proxy
    '''
    def setup_loader_modules(self):
        patcher = patch('salt.utils.vmware.get_service_instance', MagicMock())
        patcher.start()
        self.addCleanup(patcher.stop)
        return {
            vsphere: {
                '__virtual__': MagicMock(return_value='vsphere'),
                'get_proxy_type': MagicMock(return_value='esxi'),
                '_get_proxy_connection_details': MagicMock()
            }
        }

    def test_supported_proxies(self):
        supported_proxies = ['esxi', 'esxcluster', 'esxdatacenter', 'vcenter', 'esxvm']
        for proxy_type in supported_proxies:
            with patch('salt.modules.vsphere.get_proxy_type',
                       MagicMock(return_value=proxy_type)):
                vsphere.get_service_instance_via_proxy()

    def test_get_service_instance_call(self):
        mock_connection_details = [MagicMock(), MagicMock(), MagicMock()]
        mock_get_service_instance = MagicMock()
        with patch('salt.modules.vsphere._get_proxy_connection_details',
                   MagicMock(return_value=mock_connection_details)):
            with patch('salt.utils.vmware.get_service_instance',
                       mock_get_service_instance):
                vsphere.get_service_instance_via_proxy()
        mock_get_service_instance.assert_called_once_with(
            *mock_connection_details)

    def test_output(self):
        mock_si = MagicMock()
        with patch('salt.utils.vmware.get_service_instance',
                   MagicMock(return_value=mock_si)):
            res = vsphere.get_service_instance_via_proxy()
        self.assertEqual(res, mock_si)


@skipIf(NO_MOCK, NO_MOCK_REASON)
class DisconnectTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Tests for salt.modules.vsphere.disconnect
    '''
    def setup_loader_modules(self):
        self.mock_si = MagicMock()
        self.addCleanup(delattr, self, 'mock_si')
        patcher = patch('salt.utils.vmware.disconnect', MagicMock())
        patcher.start()
        self.addCleanup(patcher.stop)
        return {
            vsphere: {
                '__virtual__': MagicMock(return_value='vsphere'),
                '_get_proxy_connection_details': MagicMock(),
                'get_proxy_type': MagicMock(return_value='esxi')
            }
        }

    def test_supported_proxies(self):
        supported_proxies = ['esxi', 'esxcluster', 'esxdatacenter', 'vcenter', 'esxvm']
        for proxy_type in supported_proxies:
            with patch('salt.modules.vsphere.get_proxy_type',
                       MagicMock(return_value=proxy_type)):
                vsphere.disconnect(self.mock_si)

    def test_disconnect_call(self):
        mock_disconnect = MagicMock()
        with patch('salt.utils.vmware.disconnect', mock_disconnect):
            vsphere.disconnect(self.mock_si)
        mock_disconnect.assert_called_once_with(self.mock_si)

    def test_output(self):
        res = vsphere.disconnect(self.mock_si)
        self.assertEqual(res, True)


@skipIf(NO_MOCK, NO_MOCK_REASON)
class TestVcenterConnectionTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Tests for salt.modules.vsphere.test_vcenter_connection
    '''
    def setup_loader_modules(self):
        self.mock_si = MagicMock()
        self.addCleanup(delattr, self, 'mock_si')
        patcher = patch('salt.utils.vmware.get_service_instance', MagicMock(return_value=self.mock_si))
        patcher.start()
        self.addCleanup(patcher.stop)
        patcher = patch('salt.utils.vmware.disconnect', MagicMock())
        patcher.start()
        self.addCleanup(patcher.stop)
        patcher = patch('salt.utils.vmware.is_connection_to_a_vcenter', MagicMock())
        patcher.start()
        self.addCleanup(patcher.stop)
        return {
            vsphere: {
                '__virtual__': MagicMock(return_value='vsphere'),
                '_get_proxy_connection_details': MagicMock(),
                'get_proxy_type': MagicMock(return_value='esxi')
            }
        }

    def test_supported_proxies(self):
        supported_proxies = ['esxi', 'esxcluster', 'esxdatacenter', 'vcenter', 'esxvm']
        for proxy_type in supported_proxies:
            with patch('salt.modules.vsphere.get_proxy_type',
                       MagicMock(return_value=proxy_type)):
                vsphere.test_vcenter_connection()

    def test_is_connection_to_a_vcenter_call_default_service_instance(self):
        mock_is_connection_to_a_vcenter = MagicMock()
        with patch('salt.utils.vmware.is_connection_to_a_vcenter',
                   mock_is_connection_to_a_vcenter):
            vsphere.test_vcenter_connection()
        mock_is_connection_to_a_vcenter.assert_called_once_with(self.mock_si)

    def test_is_connection_to_a_vcenter_call_explicit_service_instance(self):
        expl_mock_si = MagicMock()
        mock_is_connection_to_a_vcenter = MagicMock()
        with patch('salt.utils.vmware.is_connection_to_a_vcenter',
                   mock_is_connection_to_a_vcenter):
            vsphere.test_vcenter_connection(expl_mock_si)
        mock_is_connection_to_a_vcenter.assert_called_once_with(expl_mock_si)

    def test_is_connection_to_a_vcenter_raises_vmware_salt_error(self):
        exc = VMwareSaltError('VMwareSaltError')
        with patch('salt.utils.vmware.is_connection_to_a_vcenter',
                   MagicMock(side_effect=exc)):
            res = vsphere.test_vcenter_connection()
        self.assertEqual(res, False)

    def test_is_connection_to_a_vcenter_raises_non_vmware_salt_error(self):
        exc = Exception('NonVMwareSaltError')
        with patch('salt.utils.vmware.is_connection_to_a_vcenter',
                   MagicMock(side_effect=exc)):
            with self.assertRaises(Exception) as excinfo:
                res = vsphere.test_vcenter_connection()
        self.assertEqual('NonVMwareSaltError', text_type(excinfo.exception))

    def test_output_true(self):
        with patch('salt.utils.vmware.is_connection_to_a_vcenter',
                   MagicMock(return_value=True)):
            res = vsphere.test_vcenter_connection()
        self.assertEqual(res, True)

    def test_output_false(self):
        with patch('salt.utils.vmware.is_connection_to_a_vcenter',
                   MagicMock(return_value=False)):
            res = vsphere.test_vcenter_connection()
        self.assertEqual(res, False)


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_PYVMOMI, 'The \'pyvmomi\' library is missing')
class ListDatacentersViaProxyTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Tests for salt.modules.vsphere.list_datacenters_via_proxy
    '''
    def setup_loader_modules(self):
        self.mock_si = MagicMock()
        self.addCleanup(delattr, self, 'mock_si')
        patcher = patch('salt.utils.vmware.get_service_instance',
                        MagicMock(return_value=self.mock_si))
        patcher.start()
        self.addCleanup(patcher.stop)
        patcher = patch('salt.utils.vmware.get_datacenters', MagicMock())
        patcher.start()
        self.addCleanup(patcher.stop)
        patcher = patch('salt.utils.vmware.get_managed_object_name',
                        MagicMock())
        patcher.start()
        self.addCleanup(patcher.stop)
        return {
            vsphere: {
                '__virtual__': MagicMock(return_value='vsphere'),
                '_get_proxy_connection_details': MagicMock(),
                'get_proxy_type': MagicMock(return_value='esxdatacenter')
            }
        }

    def test_supported_proxies(self):
        supported_proxies = ['esxcluster', 'esxdatacenter', 'vcenter', 'esxvm']
        for proxy_type in supported_proxies:
            with patch('salt.modules.vsphere.get_proxy_type',
                       MagicMock(return_value=proxy_type)):
                vsphere.list_datacenters_via_proxy()

    def test_default_params(self):
        mock_get_datacenters = MagicMock()
        with patch('salt.utils.vmware.get_datacenters',
                   mock_get_datacenters):
            vsphere.list_datacenters_via_proxy()
        mock_get_datacenters.assert_called_once_with(self.mock_si,
                                                     get_all_datacenters=True)

    def test_defined_service_instance(self):
        mock_si = MagicMock()
        mock_get_datacenters = MagicMock()
        with patch('salt.utils.vmware.get_datacenters',
                   mock_get_datacenters):
            vsphere.list_datacenters_via_proxy(service_instance=mock_si)
        mock_get_datacenters.assert_called_once_with(mock_si,
                                                     get_all_datacenters=True)

    def test_defined_datacenter_names(self):
        mock_datacenters = MagicMock()
        mock_get_datacenters = MagicMock()
        with patch('salt.utils.vmware.get_datacenters',
                   mock_get_datacenters):
            vsphere.list_datacenters_via_proxy(mock_datacenters)
        mock_get_datacenters.assert_called_once_with(self.mock_si,
                                                     mock_datacenters)

    def test_get_managed_object_name_calls(self):
        mock_get_managed_object_name = MagicMock()
        mock_dcs = [MagicMock(), MagicMock()]
        with patch('salt.utils.vmware.get_datacenters',
                   MagicMock(return_value=mock_dcs)):
            with patch('salt.utils.vmware.get_managed_object_name',
                       mock_get_managed_object_name):
                vsphere.list_datacenters_via_proxy()
        mock_get_managed_object_name.assert_has_calls([call(mock_dcs[0]),
                                                       call(mock_dcs[1])])

    def test_returned_array(self):
        with patch('salt.utils.vmware.get_datacenters',
                   MagicMock(return_value=[MagicMock(), MagicMock()])):
            # 2 datacenters
            with patch('salt.utils.vmware.get_managed_object_name',
                       MagicMock(side_effect=['fake_dc1', 'fake_dc2',
                                              'fake_dc3'])):
                # 3 possible names
                res = vsphere.list_datacenters_via_proxy()

        # Just the first two names are in the result
        self.assertEqual(res, [{'name': 'fake_dc1'}, {'name': 'fake_dc2'}])


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_PYVMOMI, 'The \'pyvmomi\' library is missing')
class CreateDatacenterTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Tests for salt.modules.vsphere.create_datacenter
    '''
    def setup_loader_modules(self):
        self.mock_si = MagicMock()
        self.addCleanup(delattr, self, 'mock_si')
        patcher = patch('salt.utils.vmware.get_service_instance', MagicMock(return_value=self.mock_si))
        patcher.start()
        self.addCleanup(patcher.stop)
        patcher = patch('salt.utils.vmware.create_datacenter', MagicMock())
        patcher.start()
        self.addCleanup(patcher.stop)
        return {
            vsphere: {
                '__virtual__': MagicMock(return_value='vsphere'),
                '_get_proxy_connection_details': MagicMock(),
                'get_proxy_type': MagicMock(return_value='esxdatacenter')
            }
        }

    def test_supported_proxies(self):
        supported_proxies = ['esxdatacenter', 'vcenter']
        for proxy_type in supported_proxies:
            with patch('salt.modules.vsphere.get_proxy_type',
                       MagicMock(return_value=proxy_type)):
                vsphere.create_datacenter('fake_dc1')

    def test_default_service_instance(self):
        mock_create_datacenter = MagicMock()
        with patch('salt.utils.vmware.create_datacenter',
                   mock_create_datacenter):
            vsphere.create_datacenter('fake_dc1')
        mock_create_datacenter.assert_called_once_with(self.mock_si,
                                                       'fake_dc1')

    def test_defined_service_instance(self):
        mock_si = MagicMock()
        mock_create_datacenter = MagicMock()
        with patch('salt.utils.vmware.create_datacenter',
                   mock_create_datacenter):
            vsphere.create_datacenter('fake_dc1', service_instance=mock_si)
        mock_create_datacenter.assert_called_once_with(mock_si, 'fake_dc1')

    def test_returned_value(self):
        res = vsphere.create_datacenter('fake_dc1')
        self.assertEqual(res, {'create_datacenter': True})


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_PYVMOMI, 'The \'pyvmomi\' library is missing')
class EraseDiskPartitionsTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Tests for salt.modules.vsphere.erase_disk_partitions
    '''
    def setup_loader_modules(self):
        return {
            vsphere: {
                '__virtual__': MagicMock(return_value='vsphere'),
                '_get_proxy_connection_details': MagicMock(),
                '__proxy__': {'esxi.get_details': MagicMock(
                    return_value={'esxi_host': 'fake_host'})}
            }
        }

    def setUp(self):
        attrs = (('mock_si', MagicMock()),
                 ('mock_host', MagicMock()))
        for attr, mock_obj in attrs:
            setattr(self, attr, mock_obj)
            self.addCleanup(delattr, self, attr)
        attrs = (('mock_proxy_target', MagicMock(return_value=self.mock_host)),
                 ('mock_erase_disk_partitions', MagicMock()))
        for attr, mock_obj in attrs:
            setattr(self, attr, mock_obj)
            self.addCleanup(delattr, self, attr)

        patches = (
            ('salt.utils.vmware.get_service_instance',
             MagicMock(return_value=self.mock_si)),
            ('salt.modules.vsphere.get_proxy_type',
             MagicMock(return_value='esxi')),
            ('salt.modules.vsphere._get_proxy_target',
             MagicMock(return_value=self.mock_host)),
            ('salt.utils.vmware.erase_disk_partitions',
             self.mock_erase_disk_partitions))
        for module, mock_obj in patches:
            patcher = patch(module, mock_obj)
            patcher.start()
            self.addCleanup(patcher.stop)

    def test_supported_proxies(self):
        supported_proxies = ['esxi']
        for proxy_type in supported_proxies:
            with patch('salt.modules.vsphere.get_proxy_type',
                       MagicMock(return_value=proxy_type)):
                vsphere.erase_disk_partitions(disk_id='fake_disk')

    def test_no_disk_id_or_scsi_address(self):
        with self.assertRaises(ArgumentValueError) as excinfo:
            vsphere.erase_disk_partitions()
        self.assertEqual('Either \'disk_id\' or \'scsi_address\' needs to '
                         'be specified', excinfo.exception.strerror)

    def test_get_proxy_target(self):
        mock_test_proxy_target = MagicMock()
        with patch('salt.modules.vsphere._get_proxy_target',
                   mock_test_proxy_target):
            vsphere.erase_disk_partitions(disk_id='fake_disk')
        mock_test_proxy_target.assert_called_once_with(self.mock_si)

    def test_scsi_address_not_found(self):
        mock = MagicMock(return_value={'bad_scsi_address': 'bad_disk_id'})
        with patch('salt.utils.vmware.get_scsi_address_to_lun_map', mock):
            with self.assertRaises(VMwareObjectRetrievalError) as excinfo:
                vsphere.erase_disk_partitions(scsi_address='fake_scsi_address')
        self.assertEqual('Scsi lun with address \'fake_scsi_address\' was '
                         'not found on host \'fake_host\'',
                         excinfo.exception.strerror)

    def test_scsi_address_to_disk_id_map(self):
        mock_disk_id = MagicMock(canonicalName='fake_scsi_disk_id')
        mock_get_scsi_addr_to_lun = \
            MagicMock(return_value={'fake_scsi_address': mock_disk_id})
        with patch('salt.utils.vmware.get_scsi_address_to_lun_map',
                   mock_get_scsi_addr_to_lun):
            vsphere.erase_disk_partitions(scsi_address='fake_scsi_address')
        mock_get_scsi_addr_to_lun.assert_called_once_with(self.mock_host)
        self.mock_erase_disk_partitions.assert_called_once_with(
            self.mock_si, self.mock_host, 'fake_scsi_disk_id',
            hostname='fake_host')

    def test_erase_disk_partitions(self):
        vsphere.erase_disk_partitions(disk_id='fake_disk_id')
        self.mock_erase_disk_partitions.assert_called_once_with(
            self.mock_si, self.mock_host, 'fake_disk_id', hostname='fake_host')


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_PYVMOMI, 'The \'pyvmomi\' library is missing')
class RemoveDatastoreTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Tests for salt.modules.vsphere.remove_datastore
    '''
    def setup_loader_modules(self):
        return {
            vsphere: {
                '__virtual__': MagicMock(return_value='vsphere'),
                '_get_proxy_connection_details': MagicMock(),
                'get_proxy_type': MagicMock(return_value='esxdatacenter'),
            }
        }

    def setUp(self):
        attrs = (('mock_si', MagicMock()),
                 ('mock_target', MagicMock()),
                 ('mock_ds', MagicMock()))
        for attr, mock_obj in attrs:
            setattr(self, attr, mock_obj)
            self.addCleanup(delattr, self, attr)

        patches = (
            ('salt.utils.vmware.get_service_instance',
             MagicMock(return_value=self.mock_si)),
            ('salt.modules.vsphere.get_proxy_type',
             MagicMock(return_value='esxdatacenter')),
            ('salt.modules.vsphere._get_proxy_target',
             MagicMock(return_value=self.mock_target)),
            ('salt.utils.vmware.get_datastores',
             MagicMock(return_value=[self.mock_ds])),
            ('salt.utils.vmware.remove_datastore', MagicMock()))
        for module, mock_obj in patches:
            patcher = patch(module, mock_obj)
            patcher.start()
            self.addCleanup(patcher.stop)

    def test_supported_proxes(self):
        supported_proxies = ['esxi', 'esxcluster', 'esxdatacenter']
        for proxy_type in supported_proxies:
            with patch('salt.modules.vsphere.get_proxy_type',
                       MagicMock(return_value=proxy_type)):
                vsphere.remove_datastore(datastore='fake_ds_name')

    def test__get_proxy_target_call(self):
        mock__get_proxy_target = MagicMock(return_value=self.mock_target)
        with patch('salt.modules.vsphere._get_proxy_target',
                   mock__get_proxy_target):
            vsphere.remove_datastore(datastore='fake_ds_name')
        mock__get_proxy_target.assert_called_once_with(self.mock_si)

    def test_get_datastores_call(self):
        mock_get_datastores = MagicMock()
        with patch('salt.utils.vmware.get_datastores',
                   mock_get_datastores):
            vsphere.remove_datastore(datastore='fake_ds')
        mock_get_datastores.assert_called_once_with(
            self.mock_si, reference=self.mock_target,
            datastore_names=['fake_ds'])

    def test_datastore_not_found(self):
        with patch('salt.utils.vmware.get_datastores',
                   MagicMock(return_value=[])):
            with self.assertRaises(VMwareObjectRetrievalError) as excinfo:
                vsphere.remove_datastore(datastore='fake_ds')
        self.assertEqual('Datastore \'fake_ds\' was not found',
                         excinfo.exception.strerror)

    def test_multiple_datastores_found(self):
        with patch('salt.utils.vmware.get_datastores',
                   MagicMock(return_value=[MagicMock(), MagicMock()])):
            with self.assertRaises(VMwareObjectRetrievalError) as excinfo:
                vsphere.remove_datastore(datastore='fake_ds')
        self.assertEqual('Multiple datastores \'fake_ds\' were found',
                         excinfo.exception.strerror)

    def test_remove_datastore_call(self):
        mock_remove_datastore = MagicMock()
        with patch('salt.utils.vmware.remove_datastore',
                   mock_remove_datastore):
            vsphere.remove_datastore(datastore='fake_ds')
        mock_remove_datastore.assert_called_once_with(
            self.mock_si, self.mock_ds)

    def test_success_output(self):
        res = vsphere.remove_datastore(datastore='fake_ds')
        self.assertTrue(res)


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_PYVMOMI, 'The \'pyvmomi\' library is missing')
class RemoveDiskgroupTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Tests for salt.modules.vsphere.remove_diskgroup
    '''
    def setup_loader_modules(self):
        return {
            vsphere: {
                '__virtual__': MagicMock(return_value='vsphere'),
                '_get_proxy_connection_details': MagicMock(),
                '__proxy__': {'esxi.get_details': MagicMock(
                    return_value={'esxi_host': 'fake_host'})}
            }
        }

    def setUp(self):
        attrs = (('mock_si', MagicMock()),
                 ('mock_host', MagicMock()),
                 ('mock_diskgroup', MagicMock()))
        for attr, mock_obj in attrs:
            setattr(self, attr, mock_obj)
            self.addCleanup(delattr, self, attr)

        patches = (
            ('salt.utils.vmware.get_service_instance',
             MagicMock(return_value=self.mock_si)),
            ('salt.modules.vsphere.get_proxy_type',
             MagicMock(return_value='esxi')),
            ('salt.modules.vsphere._get_proxy_target',
             MagicMock(return_value=self.mock_host)),
            ('salt.utils.vmware.get_diskgroups',
             MagicMock(return_value=[self.mock_diskgroup])),
            ('salt.utils.vsan.remove_diskgroup', MagicMock()))
        for module, mock_obj in patches:
            patcher = patch(module, mock_obj)
            patcher.start()
            self.addCleanup(patcher.stop)

    def test_supported_proxes(self):
        supported_proxies = ['esxi']
        for proxy_type in supported_proxies:
            with patch('salt.modules.vsphere.get_proxy_type',
                       MagicMock(return_value=proxy_type)):
                vsphere.remove_diskgroup(cache_disk_id='fake_disk_id')

    def test__get_proxy_target_call(self):
        mock__get_proxy_target = MagicMock(return_value=self.mock_host)
        with patch('salt.modules.vsphere._get_proxy_target',
                   mock__get_proxy_target):
            vsphere.remove_diskgroup(cache_disk_id='fake_disk_id')
        mock__get_proxy_target.assert_called_once_with(self.mock_si)

    def test_get_disk_groups(self):
        mock_get_diskgroups = MagicMock(return_value=[self.mock_diskgroup])
        with patch('salt.utils.vmware.get_diskgroups',
                   mock_get_diskgroups):
            vsphere.remove_diskgroup(cache_disk_id='fake_disk_id')
        mock_get_diskgroups.assert_called_once_with(
            self.mock_host, cache_disk_ids=['fake_disk_id'])

    def test_disk_group_not_found_safety_checks_set(self):
        with patch('salt.utils.vmware.get_diskgroups',
                   MagicMock(return_value=[])):
            with self.assertRaises(VMwareObjectRetrievalError) as excinfo:
                vsphere.remove_diskgroup(cache_disk_id='fake_disk_id')
        self.assertEqual('No diskgroup with cache disk id '
                         '\'fake_disk_id\' was found in ESXi host '
                         '\'fake_host\'',
                         excinfo.exception.strerror)

    def test_remove_disk_group(self):
        mock_remove_diskgroup = MagicMock(return_value=None)
        with patch('salt.utils.vsan.remove_diskgroup',
                   mock_remove_diskgroup):
            vsphere.remove_diskgroup(cache_disk_id='fake_disk_id')
        mock_remove_diskgroup.assert_called_once_with(
            self.mock_si, self.mock_host, self.mock_diskgroup,
            data_accessibility=True)

    def test_remove_disk_group_data_accessibility_false(self):
        mock_remove_diskgroup = MagicMock(return_value=None)
        with patch('salt.utils.vsan.remove_diskgroup',
                   mock_remove_diskgroup):
            vsphere.remove_diskgroup(cache_disk_id='fake_disk_id',
                                     data_accessibility=False)
        mock_remove_diskgroup.assert_called_once_with(
            self.mock_si, self.mock_host, self.mock_diskgroup,
            data_accessibility=False)

    def test_success_output(self):
        res = vsphere.remove_diskgroup(cache_disk_id='fake_disk_id')
        self.assertTrue(res)


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_PYVMOMI, 'The \'pyvmomi\' library is missing')
@skipIf(not vsphere.HAS_JSONSCHEMA, 'The \'jsonschema\' library is missing')
class RemoveCapacityFromDiskgroupTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Tests for salt.modules.vsphere.remove_capacity_from_diskgroup
    '''
    def setup_loader_modules(self):
        return {
            vsphere: {
                '__virtual__': MagicMock(return_value='vsphere'),
                '_get_proxy_connection_details': MagicMock(),
                '__proxy__': {'esxi.get_details': MagicMock(
                    return_value={'esxi_host': 'fake_host'})}
            }
        }

    def setUp(self):
        attrs = (('mock_si', MagicMock()),
                 ('mock_schema', MagicMock()),
                 ('mock_host', MagicMock()),
                 ('mock_disk1', MagicMock(canonicalName='fake_disk1')),
                 ('mock_disk2', MagicMock(canonicalName='fake_disk2')),
                 ('mock_disk3', MagicMock(canonicalName='fake_disk3')),
                 ('mock_diskgroup', MagicMock()))
        for attr, mock_obj in attrs:
            setattr(self, attr, mock_obj)
            self.addCleanup(delattr, self, attr)

        patches = (
            ('salt.utils.vmware.get_service_instance',
             MagicMock(return_value=self.mock_si)),
            ('salt.modules.vsphere.DiskGroupsDiskIdSchema.serialize',
             MagicMock(return_value=self.mock_schema)),
            ('salt.modules.vsphere.jsonschema.validate', MagicMock()),
            ('salt.modules.vsphere.get_proxy_type',
             MagicMock(return_value='esxi')),
            ('salt.modules.vsphere._get_proxy_target',
             MagicMock(return_value=self.mock_host)),
            ('salt.utils.vmware.get_disks',
             MagicMock(return_value=[self.mock_disk1, self.mock_disk2,
                                     self.mock_disk3])),
            ('salt.utils.vmware.get_diskgroups',
             MagicMock(return_value=[self.mock_diskgroup])),
            ('salt.utils.vsan.remove_capacity_from_diskgroup', MagicMock()))
        for module, mock_obj in patches:
            patcher = patch(module, mock_obj)
            patcher.start()
            self.addCleanup(patcher.stop)

    def test_validate(self):
        mock_schema_validate = MagicMock()
        with patch('salt.modules.vsphere.jsonschema.validate',
                   mock_schema_validate):
            vsphere.remove_capacity_from_diskgroup(
                cache_disk_id='fake_cache_disk_id',
                capacity_disk_ids=['fake_disk1', 'fake_disk2'])
        mock_schema_validate.assert_called_once_with(
            {'diskgroups': [{'cache_id': 'fake_cache_disk_id',
                             'capacity_ids': ['fake_disk1',
                                              'fake_disk2']}]},
            self.mock_schema)

    def test_invalid_schema_validation(self):
        mock_schema_validate = MagicMock(
            side_effect=vsphere.jsonschema.exceptions.ValidationError('err'))
        with patch('salt.modules.vsphere.jsonschema.validate',
                   mock_schema_validate):
            with self.assertRaises(ArgumentValueError) as excinfo:
                vsphere.remove_capacity_from_diskgroup(
                    cache_disk_id='fake_cache_disk_id',
                    capacity_disk_ids=['fake_disk1', 'fake_disk2'])
        self.assertEqual('err', excinfo.exception.strerror)

    def test_supported_proxes(self):
        supported_proxies = ['esxi']
        for proxy_type in supported_proxies:
            with patch('salt.modules.vsphere.get_proxy_type',
                       MagicMock(return_value=proxy_type)):
                vsphere.remove_capacity_from_diskgroup(
                    cache_disk_id='fake_cache_disk_id',
                    capacity_disk_ids=['fake_disk1', 'fake_disk2'])

    def test__get_proxy_target_call(self):
        mock__get_proxy_target = MagicMock(return_value=self.mock_host)
        with patch('salt.modules.vsphere._get_proxy_target',
                   mock__get_proxy_target):
            vsphere.remove_capacity_from_diskgroup(
                cache_disk_id='fake_cache_disk_id',
                capacity_disk_ids=['fake_disk1', 'fake_disk2'])
        mock__get_proxy_target.assert_called_once_with(self.mock_si)

    def test_get_disks(self):
        mock_get_disks = MagicMock(
            return_value=[self.mock_disk1, self.mock_disk2, self.mock_disk3])
        with patch('salt.utils.vmware.get_disks', mock_get_disks):
            vsphere.remove_capacity_from_diskgroup(
                cache_disk_id='fake_cache_disk_id',
                capacity_disk_ids=['fake_disk1', 'fake_disk2'])
        mock_get_disks.assert_called_once_with(
            self.mock_host, disk_ids=['fake_disk1', 'fake_disk2'])

    def test_disk_not_found_safety_checks_set(self):
        mock_get_disks = MagicMock(
            return_value=[self.mock_disk1, self.mock_disk2, self.mock_disk3])
        with patch('salt.utils.vmware.get_disks', mock_get_disks):
            with self.assertRaises(VMwareObjectRetrievalError) as excinfo:
                vsphere.remove_capacity_from_diskgroup(
                    cache_disk_id='fake_cache_disk_id',
                    capacity_disk_ids=['fake_disk1', 'fake_disk4'],
                    safety_checks=True)
        self.assertEqual('No disk with id \'fake_disk4\' was found '
                         'in ESXi host \'fake_host\'',
                         excinfo.exception.strerror)

    def test_get_diskgroups(self):
        mock_get_diskgroups = MagicMock(return_value=[self.mock_diskgroup])
        with patch('salt.utils.vmware.get_diskgroups',
                   mock_get_diskgroups):
            vsphere.remove_capacity_from_diskgroup(
                cache_disk_id='fake_cache_disk_id',
                capacity_disk_ids=['fake_disk1', 'fake_disk2'])
        mock_get_diskgroups.assert_called_once_with(
            self.mock_host, cache_disk_ids=['fake_cache_disk_id'])

    def test_diskgroup_not_found(self):
        with patch('salt.utils.vmware.get_diskgroups',
                   MagicMock(return_value=[])):
            with self.assertRaises(VMwareObjectRetrievalError) as excinfo:
                vsphere.remove_capacity_from_diskgroup(
                    cache_disk_id='fake_cache_disk_id',
                    capacity_disk_ids=['fake_disk1', 'fake_disk2'])
        self.assertEqual('No diskgroup with cache disk id '
                         '\'fake_cache_disk_id\' was found in ESXi host '
                         '\'fake_host\'',
                         excinfo.exception.strerror)

    def test_remove_capacity_from_diskgroup(self):
        mock_remove_capacity_from_diskgroup = MagicMock()
        with patch('salt.utils.vsan.remove_capacity_from_diskgroup',
                   mock_remove_capacity_from_diskgroup):
            vsphere.remove_capacity_from_diskgroup(
                cache_disk_id='fake_cache_disk_id',
                capacity_disk_ids=['fake_disk1', 'fake_disk2'])
        mock_remove_capacity_from_diskgroup.assert_called_once_with(
            self.mock_si, self.mock_host, self.mock_diskgroup,
            capacity_disks=[self.mock_disk1, self.mock_disk2],
            data_evacuation=True)

    def test_remove_capacity_from_diskgroup_data_evacuation_false(self):
        mock_remove_capacity_from_diskgroup = MagicMock()
        with patch('salt.utils.vsan.remove_capacity_from_diskgroup',
                   mock_remove_capacity_from_diskgroup):
            vsphere.remove_capacity_from_diskgroup(
                cache_disk_id='fake_cache_disk_id',
                capacity_disk_ids=['fake_disk1', 'fake_disk2'],
                data_evacuation=False)
        mock_remove_capacity_from_diskgroup.assert_called_once_with(
            self.mock_si, self.mock_host, self.mock_diskgroup,
            capacity_disks=[self.mock_disk1, self.mock_disk2],
            data_evacuation=False)

    def test_success_output(self):
        res = vsphere.remove_capacity_from_diskgroup(
            cache_disk_id='fake_cache_disk_id',
            capacity_disk_ids=['fake_disk1', 'fake_disk2'])
        self.assertTrue(res)


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_PYVMOMI, 'The \'pyvmomi\' library is missing')
class ListClusterTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Tests for salt.modules.vsphere.list_cluster
    '''
    def setup_loader_modules(self):
        return {
            vsphere: {
                '__virtual__': MagicMock(return_value='vsphere'),
                '_get_proxy_connection_details': MagicMock(),
                '__salt__': {}
            }
        }

    def setUp(self):
        attrs = (('mock_si', MagicMock()),
                 ('mock_dc', MagicMock()),
                 ('mock_cl', MagicMock()),
                 ('mock__get_cluster_dict', MagicMock()))
        for attr, mock_obj in attrs:
            setattr(self, attr, mock_obj)
            self.addCleanup(delattr, self, attr)
        attrs = (('mock_get_cluster', MagicMock(return_value=self.mock_cl)),)
        for attr, mock_obj in attrs:
            setattr(self, attr, mock_obj)
            self.addCleanup(delattr, self, attr)
        patches = (
            ('salt.utils.vmware.get_service_instance',
             MagicMock(return_value=self.mock_si)),
            ('salt.modules.vsphere.get_proxy_type',
             MagicMock(return_value='esxcluster')),
            ('salt.modules.vsphere._get_proxy_target',
             MagicMock(return_value=self.mock_cl)),
            ('salt.utils.vmware.get_cluster', self.mock_get_cluster),
            ('salt.modules.vsphere._get_cluster_dict',
             self.mock__get_cluster_dict))
        for module, mock_obj in patches:
            patcher = patch(module, mock_obj)
            patcher.start()
            self.addCleanup(patcher.stop)
        # Patch __salt__ dunder
        patcher = patch.dict(vsphere.__salt__,
                             {'esxcluster.get_details':
                              MagicMock(return_value={'cluster': 'cl'})})
        patcher.start()
        self.addCleanup(patcher.stop)

    def test_supported_proxies(self):
        supported_proxies = ['esxcluster', 'esxdatacenter']
        for proxy_type in supported_proxies:
            with patch('salt.modules.vsphere.get_proxy_type',
                       MagicMock(return_value=proxy_type)):
                vsphere.list_cluster(cluster='cl')

    def test_default_service_instance(self):
        mock__get_proxy_target = MagicMock()
        with patch('salt.modules.vsphere._get_proxy_target',
                   mock__get_proxy_target):
            vsphere.list_cluster()
        mock__get_proxy_target.assert_called_once_with(self.mock_si)

    def test_defined_service_instance(self):
        mock_si = MagicMock()
        mock__get_proxy_target = MagicMock()
        with patch('salt.modules.vsphere._get_proxy_target',
                   mock__get_proxy_target):
            vsphere.list_cluster(service_instance=mock_si)
        mock__get_proxy_target.assert_called_once_with(mock_si)

    def test_no_cluster_raises_argument_value_error(self):
        with patch('salt.modules.vsphere.get_proxy_type',
                   MagicMock(return_value='esxdatacenter')):
            with patch('salt.modules.vsphere._get_proxy_target', MagicMock()):
                with self.assertRaises(ArgumentValueError) as excinfo:
                    vsphere.list_cluster()
        self.assertEqual(excinfo.exception.strerror,
                         '\'cluster\' needs to be specified')

    def test_get_cluster_call(self):
        with patch('salt.modules.vsphere.get_proxy_type',
                   MagicMock(return_value='esxdatacenter')):
            with patch('salt.modules.vsphere._get_proxy_target',
                       MagicMock(return_value=self.mock_dc)):
                vsphere.list_cluster(cluster='cl')
        self.mock_get_cluster.assert_called_once_with(self.mock_dc, 'cl')

    def test__get_cluster_dict_call(self):
        vsphere.list_cluster()
        self.mock__get_cluster_dict.assert_called_once_with('cl', self.mock_cl)


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_PYVMOMI, 'The \'pyvmomi\' library is missing')
class RenameDatastoreTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Tests for salt.modules.vsphere.rename_datastore
    '''
    def setup_loader_modules(self):
        return {
            vsphere: {
                '__virtual__': MagicMock(return_value='vsphere'),
                '_get_proxy_connection_details': MagicMock(),
                'get_proxy_type': MagicMock(return_value='esxdatacenter')
            }
        }

    def setUp(self):
        self.mock_si = MagicMock()
        self.mock_target = MagicMock()
        self.mock_ds_ref = MagicMock()
        self.mock_get_datastores = MagicMock(return_value=[self.mock_ds_ref])
        self.mock_rename_datastore = MagicMock()
        patches = (
            ('salt.utils.vmware.get_service_instance',
             MagicMock(return_value=self.mock_si)),
            ('salt.modules.vsphere._get_proxy_target',
             MagicMock(return_value=self.mock_target)),
            ('salt.utils.vmware.get_datastores',
             self.mock_get_datastores),
            ('salt.utils.vmware.rename_datastore',
             self.mock_rename_datastore))
        for mod, mock in patches:
            patcher = patch(mod, mock)
            patcher.start()
            self.addCleanup(patcher.stop)

    def tearDown(self):
        for attr in ('mock_si', 'mock_target', 'mock_ds_ref',
                     'mock_get_datastores', 'mock_rename_datastore'):
            delattr(self, attr)

    def test_supported_proxes(self):
        supported_proxies = ['esxi', 'esxcluster', 'esxdatacenter']
        for proxy_type in supported_proxies:
            with patch('salt.modules.vsphere.get_proxy_type',
                       MagicMock(return_value=proxy_type)):
                vsphere.rename_datastore('current_ds_name', 'new_ds_name')

    def test_default_service_instance(self):
        mock__get_proxy_target = MagicMock()
        with patch('salt.modules.vsphere._get_proxy_target',
                   mock__get_proxy_target):
            vsphere.rename_datastore('current_ds_name', 'new_ds_name')
        mock__get_proxy_target.assert_called_once_with(self.mock_si)

    def test_defined_service_instance(self):
        mock_si = MagicMock()
        mock__get_proxy_target = MagicMock()
        with patch('salt.modules.vsphere._get_proxy_target',
                   mock__get_proxy_target):
            vsphere.rename_datastore('current_ds_name', 'new_ds_name',
                                     service_instance=mock_si)

        mock__get_proxy_target.assert_called_once_with(mock_si)

    def test_get_datastore_call(self):
        vsphere.rename_datastore('current_ds_name', 'new_ds_name')
        self.mock_get_datastores.assert_called_once_with(
            self.mock_si, self.mock_target,
            datastore_names=['current_ds_name'])

    def test_get_no_datastores(self):
        with patch('salt.utils.vmware.get_datastores',
                   MagicMock(return_value=[])):
            with self.assertRaises(VMwareObjectRetrievalError) as excinfo:
                vsphere.rename_datastore('current_ds_name', 'new_ds_name')
        self.assertEqual(excinfo.exception.strerror,
                         'Datastore \'current_ds_name\' was not found')

    def test_rename_datastore_call(self):
        vsphere.rename_datastore('current_ds_name', 'new_ds_name')
        self.mock_rename_datastore.assert_called_once_with(
            self.mock_ds_ref, 'new_ds_name')


@skipIf(NO_MOCK, NO_MOCK_REASON)
class _GetProxyTargetTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Tests for salt.modules.vsphere._get_proxy_target
    '''
    def setup_loader_modules(self):
        return {
            vsphere: {
                '__virtual__': MagicMock(return_value='vsphere'),
                '_get_proxy_connection_details': MagicMock(),
                'get_proxy_type': MagicMock(return_value='esxdatacenter')
            }
        }

    def setUp(self):
        attrs = (('mock_si', MagicMock()),
                 ('mock_dc', MagicMock()),
                 ('mock_cl', MagicMock()),
                 ('mock_root', MagicMock()))
        for attr, mock_obj in attrs:
            setattr(self, attr, mock_obj)
            self.addCleanup(delattr, self, attr)
        attrs = (('mock_get_datacenter', MagicMock(return_value=self.mock_dc)),
                 ('mock_get_cluster', MagicMock(return_value=self.mock_cl)),
                 ('mock_get_root_folder',
                  MagicMock(return_value=self.mock_root)))
        for attr, mock_obj in attrs:
            setattr(self, attr, mock_obj)
            self.addCleanup(delattr, self, attr)
        patches = (
            ('salt.modules.vsphere.get_proxy_type',
             MagicMock(return_value='esxcluster')),
            ('salt.utils.vmware.is_connection_to_a_vcenter',
             MagicMock(return_value=True)),
            ('salt.modules.vsphere._get_esxcluster_proxy_details',
             MagicMock(return_value=(None, None, None, None, None, None, None,
                                     None, 'datacenter', 'cluster'))),
            ('salt.modules.vsphere._get_esxdatacenter_proxy_details',
             MagicMock(return_value=(None, None, None, None, None, None, None,
                                     None, 'datacenter'))),
            ('salt.utils.vmware.get_datacenter', self.mock_get_datacenter),
            ('salt.utils.vmware.get_cluster', self.mock_get_cluster),
            ('salt.utils.vmware.get_root_folder', self.mock_get_root_folder))
        for module, mock_obj in patches:
            patcher = patch(module, mock_obj)
            patcher.start()
            self.addCleanup(patcher.stop)

    def test_supported_proxies(self):
        supported_proxies = ['esxcluster', 'esxdatacenter']
        for proxy_type in supported_proxies:
            with patch('salt.modules.vsphere.get_proxy_type',
                       MagicMock(return_value=proxy_type)):
                vsphere._get_proxy_target(self.mock_si)

    def test_connected_to_esxi(self):
        with patch('salt.utils.vmware.is_connection_to_a_vcenter',
                   MagicMock(return_value=False)):
            with self.assertRaises(CommandExecutionError) as excinfo:
                vsphere._get_proxy_target(self.mock_si)
            self.assertEqual(excinfo.exception.strerror,
                             '\'_get_proxy_target\' not supported when '
                             'connected via the ESXi host')

    def test_get_cluster_call(self):
        vsphere._get_proxy_target(self.mock_si)
        self.mock_get_datacenter.assert_called_once_with(self.mock_si,
                                                         'datacenter')
        self.mock_get_cluster.assert_called_once_with(self.mock_dc, 'cluster')

    def test_esxcluster_proxy_return(self):
        with patch('salt.modules.vsphere.get_proxy_type',
                   MagicMock(return_value='esxcluster')):
            ret = vsphere._get_proxy_target(self.mock_si)
        self.assertEqual(ret, self.mock_cl)

    def test_get_datacenter_call(self):
        with patch('salt.modules.vsphere.get_proxy_type',
                   MagicMock(return_value='esxdatacenter')):
            vsphere._get_proxy_target(self.mock_si)
        self.mock_get_datacenter.assert_called_once_with(self.mock_si,
                                                         'datacenter')
        self.assertEqual(self.mock_get_cluster.call_count, 0)

    def test_esxdatacenter_proxy_return(self):
        with patch('salt.modules.vsphere.get_proxy_type',
                   MagicMock(return_value='esxdatacenter')):
            ret = vsphere._get_proxy_target(self.mock_si)
        self.assertEqual(ret, self.mock_dc)

    def test_vcenter_proxy_return(self):
        with patch('salt.modules.vsphere.get_proxy_type',
                   MagicMock(return_value='vcenter')):
            ret = vsphere._get_proxy_target(self.mock_si)
        self.mock_get_root_folder.assert_called_once_with(self.mock_si)
        self.assertEqual(ret, self.mock_root)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
'''
# Import Python libs
from __future__ import absolute_import, unicode_literals, print_function
import os
import time
import tempfile

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.paths import TMP
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch
)

# Import Salt Libs
import salt.config
import salt.loader
import salt.utils.jid
import salt.utils.event
import salt.states.saltmod as saltmod


@skipIf(NO_MOCK, NO_MOCK_REASON)
class SaltmodTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.states.saltmod
    '''
    def setup_loader_modules(self):
        utils = salt.loader.utils(
            salt.config.DEFAULT_MINION_OPTS,
            whitelist=['state']
        )
        return {
            saltmod: {
                '__env__': 'base',
                '__opts__': {
                    '__role': 'master',
                    'file_client': 'remote',
                    'sock_dir': tempfile.mkdtemp(dir=TMP),
                    'transport': 'tcp'
                },
                '__salt__': {'saltutil.cmd': MagicMock()},
                '__orchestration_jid__': salt.utils.jid.gen_jid({}),
                '__utils__': utils,
            }
        }

    # 'state' function tests: 1

    def test_state(self):
        '''
        Test to invoke a state run on a given target
        '''
        name = 'state'
        tgt = 'minion1'

        comt = ('Passed invalid value for \'allow_fail\', must be an int')

        ret = {'name': name,
               'changes': {},
               'result': False,
               'comment': comt}

        test_ret = {'name': name,
                    'changes': {},
                    'result': True,
                    'comment': 'States ran successfully.'
                    }

        test_batch_return = {
            'minion1': {
                'ret': {
                    'test_|-notify_me_|-this is a name_|-show_notification': {
                        'comment': 'Notify me',
                        'name': 'this is a name',
                        'start_time': '10:43:41.487565',
                        'result': True,
                        'duration': 0.35,
                        '__run_num__': 0,
                        '__sls__': 'demo',
                        'changes': {},
                        '__id__': 'notify_me'
                    },
                    'retcode': 0
                },
                'out': 'highstate'
            },
            'minion2': {
                'ret': {
                    'test_|-notify_me_|-this is a name_|-show_notification': {
                        'comment': 'Notify me',
                        'name': 'this is a name',
                        'start_time': '10:43:41.487565',
                        'result': True,
                        'duration': 0.35,
                        '__run_num__': 0,
                        '__sls__': 'demo',
                        'changes': {},
                        '__id__': 'notify_me'
                    },
                    'retcode': 0
                },
                'out': 'highstate'
            },
            'minion3': {
                'ret': {
                    'test_|-notify_me_|-this is a name_|-show_notification': {
                        'comment': 'Notify me',
                        'name': 'this is a name',
                        'start_time': '10:43:41.487565',
                        'result': True,
                        'duration': 0.35,
                        '__run_num__': 0,
                        '__sls__': 'demo',
                        'changes': {},
                        '__id__': 'notify_me'
                    },
                    'retcode': 0
                },
                'out': 'highstate'
            }
        }

        self.assertDictEqual(saltmod.state(name, tgt, allow_fail='a'), ret)

        comt = ('No highstate or sls specified, no execution made')
        ret.update({'comment': comt})
        self.assertDictEqual(saltmod.state(name, tgt), ret)

        comt = ("Must pass in boolean for value of 'concurrent'")
        ret.update({'comment': comt})
        self.assertDictEqual(saltmod.state(name, tgt, highstate=True,
                                           concurrent='a'), ret)

        ret.update({'comment': comt, 'result': None})
        with patch.dict(saltmod.__opts__, {'test': True}):
            self.assertDictEqual(saltmod.state(name, tgt, highstate=True), test_ret)

        ret.update({'comment': 'States ran successfully. No changes made to silver.', 'result': True, '__jid__': '20170406104341210934'})
        with patch.dict(saltmod.__opts__, {'test': False}):
            mock = MagicMock(return_value={'silver': {'jid': '20170406104341210934', 'retcode': 0, 'ret': {'test_|-notify_me_|-this is a name_|-show_notification': {'comment': 'Notify me', 'name': 'this is a name', 'start_time': '10:43:41.487565', 'result': True, 'duration': 0.35, '__run_num__': 0, '__sls__': 'demo', 'changes': {}, '__id__': 'notify_me'}}, 'out': 'highstate'}})
            with patch.dict(saltmod.__salt__, {'saltutil.cmd': mock}):
                self.assertDictEqual(saltmod.state(name, tgt, highstate=True), ret)

        ret.update({'comment': 'States ran successfully. No changes made to minion1, minion3, minion2.'})
        del ret['__jid__']
        with patch.dict(saltmod.__opts__, {'test': False}):
            with patch.dict(saltmod.__salt__, {'saltutil.cmd': MagicMock(return_value=test_batch_return)}):
                state_run = saltmod.state(name, tgt, highstate=True)

                # Test return without checking the comment contents. Comments are tested later.
                comment = state_run.pop('comment')
                ret.pop('comment')
                self.assertDictEqual(state_run, ret)

                # Check the comment contents in a non-order specific way (ordering fails sometimes on PY3)
                self.assertIn('States ran successfully. No changes made to', comment)
                for minion in ['minion1', 'minion2', 'minion3']:
                    self.assertIn(minion, comment)

    # 'function' function tests: 1

    def test_function(self):
        '''
        Test to execute a single module function on a remote
        minion via salt or salt-ssh
        '''
        name = 'state'
        tgt = 'larry'

        comt = ('Function state will be executed'
                ' on target {0} as test=False'.format(tgt))

        ret = {'name': name,
               'changes': {},
               'result': None,
               'comment': comt}

        with patch.dict(saltmod.__opts__, {'test': True}):
            self.assertDictEqual(saltmod.function(name, tgt), ret)

        ret.update({'result': True,
                    'changes': {'out': 'highstate', 'ret': {tgt: ''}},
                    'comment': 'Function ran successfully.'
                              ' Function state ran on {0}.'.format(tgt)})
        with patch.dict(saltmod.__opts__, {'test': False}):
            mock_ret = {'larry': {'ret': '', 'retcode': 0, 'failed': False}}
            mock_cmd = MagicMock(return_value=mock_ret)
            with patch.dict(saltmod.__salt__, {'saltutil.cmd': mock_cmd}):
                self.assertDictEqual(saltmod.function(name, tgt), ret)

    # 'wait_for_event' function tests: 1

    def test_wait_for_event(self):
        '''
        Test to watch Salt's event bus and block until a condition is met
        '''
        name = 'state'
        tgt = 'minion1'

        comt = ('Timeout value reached.')

        ret = {'name': name,
               'changes': {},
               'result': False,
               'comment': comt}

        class Mockevent(object):
            '''
            Mock event class
            '''
            flag = None

            def __init__(self):
                self.full = None

            def get_event(self, full):
                '''
                Mock get_event method
                '''
                self.full = full
                if self.flag:
                    return {'tag': name, 'data': {}}
                return None

        with patch.object(salt.utils.event, 'get_event',
                          MagicMock(return_value=Mockevent())):
            with patch.dict(saltmod.__opts__, {'sock_dir': True,
                                               'transport': True}):
                with patch.object(time, 'time', MagicMock(return_value=1.0)):
                    self.assertDictEqual(saltmod.wait_for_event(name, 'salt',
                                                                timeout=-1.0),
                                         ret)

                    Mockevent.flag = True
                    ret.update({'comment': 'All events seen in 0.0 seconds.',
                                'result': True})
                    self.assertDictEqual(saltmod.wait_for_event(name, ''), ret)

                    ret.update({'comment': 'Timeout value reached.',
                                'result': False})
                    self.assertDictEqual(saltmod.wait_for_event(name, tgt,
                                                                timeout=-1.0),
                                         ret)

    # 'runner' function tests: 1

    def test_runner(self):
        '''
        Test to execute a runner module on the master
        '''
        name = 'state'

        ret = {'changes': {'return': True}, 'name': 'state', 'result': True,
               'comment': 'Runner function \'state\' executed.',
               '__orchestration__': True}
        runner_mock = MagicMock(return_value={'return': True})

        with patch.dict(saltmod.__salt__, {'saltutil.runner': runner_mock}):
            self.assertDictEqual(saltmod.runner(name), ret)

    # 'wheel' function tests: 1

    def test_wheel(self):
        '''
        Test to execute a wheel module on the master
        '''
        name = 'state'

        ret = {'changes': {'return': True}, 'name': 'state', 'result': True,
               'comment': 'Wheel function \'state\' executed.',
               '__orchestration__': True}
        wheel_mock = MagicMock(return_value={'return': True})

        with patch.dict(saltmod.__salt__, {'saltutil.wheel': wheel_mock}):
            self.assertDictEqual(saltmod.wheel(name), ret)

    def test_state_ssh(self):
        '''
        Test saltmod passes roster to saltutil.cmd
        '''
        origcmd = saltmod.__salt__['saltutil.cmd']
        cmd_kwargs = {}
        cmd_args = []

        def cmd_mock(*args, **kwargs):
            cmd_args.extend(args)
            cmd_kwargs.update(kwargs)
            return origcmd(*args, **kwargs)

        with patch.dict(saltmod.__salt__, {'saltutil.cmd': cmd_mock}):
            ret = saltmod.state('state.sls', tgt='*', ssh=True, highstate=True, roster='my_roster')
        assert 'roster' in cmd_kwargs
        assert cmd_kwargs['roster'] == 'my_roster'


@skipIf(NO_MOCK, NO_MOCK_REASON)
class StatemodTests(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        self.tmp_cachedir = tempfile.mkdtemp(dir=TMP)
        return {
            saltmod: {
                '__env__': 'base',
                '__opts__': {
                    'id': 'webserver2',
                    'argv': [],
                    '__role': 'master',
                    'cachedir': self.tmp_cachedir,
                    'extension_modules': os.path.join(self.tmp_cachedir, 'extmods'),
                },
                '__salt__': {'saltutil.cmd': MagicMock()},
                '__orchestration_jid__': salt.utils.jid.gen_jid({})
            }
        }

    def test_statemod_state(self):
        ''' Smoke test for for salt.states.statemod.state().  Ensures that we
            don't take an exception if optional parameters are not specified in
            __opts__ or __env__.
        '''
        args = ('webserver_setup', 'webserver2')
        kwargs = {
            'tgt_type': 'glob',
            'fail_minions': None,
            'pillar': None,
            'top': None,
            'batch': None,
            'orchestration_jid': None,
            'sls': 'vroom',
            'queue': False,
            'concurrent': False,
            'highstate': None,
            'expr_form': None,
            'ret': '',
            'ssh': False,
            'timeout': None, 'test': False,
            'allow_fail': 0,
            'saltenv': None,
            'expect_minions': False
        }
        ret = saltmod.state(*args, **kwargs)
        expected = {
            'comment': 'States ran successfully.',
            'changes': {},
            'name': 'webserver_setup',
            'result': True
        }
        self.assertEqual(ret, expected)



# -*- coding: utf-8 -*-
'''
Tests for salt.utils.data
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import logging

# Import Salt libs
import salt.utils.data
import salt.utils.stringutils
from salt.utils.odict import OrderedDict
from tests.support.unit import TestCase, skipIf, LOREM_IPSUM
from tests.support.mock import patch, NO_MOCK, NO_MOCK_REASON
from salt.ext.six.moves import builtins  # pylint: disable=import-error,redefined-builtin
from salt.ext import six

log = logging.getLogger(__name__)
_b = lambda x: x.encode('utf-8')
_s = lambda x: salt.utils.stringutils.to_str(x, normalize=True)
# Some randomized data that will not decode
BYTES = b'1\x814\x10'

# This is an example of a unicode string with й constructed using two separate
# code points. Do not modify it.
EGGS = '\u044f\u0438\u0306\u0446\u0430'


class DataTestCase(TestCase):
    test_data = [
        'unicode_str',
        _b('питон'),
        123,
        456.789,
        True,
        False,
        None,
        EGGS,
        BYTES,
        [123, 456.789, _b('спам'), True, False, None, EGGS, BYTES],
        (987, 654.321, _b('яйца'), EGGS, None, (True, EGGS, BYTES)),
        {_b('str_key'): _b('str_val'),
         None: True,
         123: 456.789,
         EGGS: BYTES,
         _b('subdict'): {'unicode_key': EGGS,
                         _b('tuple'): (123, 'hello', _b('world'), True, EGGS, BYTES),
                         _b('list'): [456, _b('спам'), False, EGGS, BYTES]}},
        OrderedDict([(_b('foo'), 'bar'), (123, 456), (EGGS, BYTES)])
    ]

    def test_sorted_ignorecase(self):
        test_list = ['foo', 'Foo', 'bar', 'Bar']
        expected_list = ['bar', 'Bar', 'foo', 'Foo']
        self.assertEqual(
            salt.utils.data.sorted_ignorecase(test_list), expected_list)

    def test_mysql_to_dict(self):
        test_mysql_output = ['+----+------+-----------+------+---------+------+-------+------------------+',
                             '| Id | User | Host      | db   | Command | Time | State | Info             |',
                             '+----+------+-----------+------+---------+------+-------+------------------+',
                             '|  7 | root | localhost | NULL | Query   |    0 | init  | show processlist |',
                             '+----+------+-----------+------+---------+------+-------+------------------+']

        ret = salt.utils.data.mysql_to_dict(test_mysql_output, 'Info')
        expected_dict = {
            'show processlist': {'Info': 'show processlist', 'db': 'NULL', 'State': 'init', 'Host': 'localhost',
                                 'Command': 'Query', 'User': 'root', 'Time': 0, 'Id': 7}}

        self.assertDictEqual(ret, expected_dict)

    def test_subdict_match(self):
        test_two_level_dict = {'foo': {'bar': 'baz'}}
        test_two_level_comb_dict = {'foo': {'bar': 'baz:woz'}}
        test_two_level_dict_and_list = {
            'abc': ['def', 'ghi', {'lorem': {'ipsum': [{'dolor': 'sit'}]}}],
        }
        test_three_level_dict = {'a': {'b': {'c': 'v'}}}

        self.assertTrue(
            salt.utils.data.subdict_match(
                test_two_level_dict, 'foo:bar:baz'
            )
        )
        # In test_two_level_comb_dict, 'foo:bar' corresponds to 'baz:woz', not
        # 'baz'. This match should return False.
        self.assertFalse(
            salt.utils.data.subdict_match(
                test_two_level_comb_dict, 'foo:bar:baz'
            )
        )
        # This tests matching with the delimiter in the value part (in other
        # words, that the path 'foo:bar' corresponds to the string 'baz:woz').
        self.assertTrue(
            salt.utils.data.subdict_match(
                test_two_level_comb_dict, 'foo:bar:baz:woz'
            )
        )
        # This would match if test_two_level_comb_dict['foo']['bar'] was equal
        # to 'baz:woz:wiz', or if there was more deep nesting. But it does not,
        # so this should return False.
        self.assertFalse(
            salt.utils.data.subdict_match(
                test_two_level_comb_dict, 'foo:bar:baz:woz:wiz'
            )
        )
        # This tests for cases when a key path corresponds to a list. The
        # value part 'ghi' should be successfully matched as it is a member of
        # the list corresponding to key path 'abc'. It is somewhat a
        # duplication of a test within test_traverse_dict_and_list, but
        # salt.utils.data.subdict_match() does more than just invoke
        # salt.utils.traverse_list_and_dict() so this particular assertion is a
        # sanity check.
        self.assertTrue(
            salt.utils.data.subdict_match(
                test_two_level_dict_and_list, 'abc:ghi'
            )
        )
        # This tests the use case of a dict embedded in a list, embedded in a
        # list, embedded in a dict. This is a rather absurd case, but it
        # confirms that match recursion works properly.
        self.assertTrue(
            salt.utils.data.subdict_match(
                test_two_level_dict_and_list, 'abc:lorem:ipsum:dolor:sit'
            )
        )
        # Test four level dict match for reference
        self.assertTrue(
            salt.utils.data.subdict_match(
                test_three_level_dict, 'a:b:c:v'
            )
        )
        self.assertFalse(
        # Test regression in 2015.8 where 'a:c:v' would match 'a:b:c:v'
            salt.utils.data.subdict_match(
                test_three_level_dict, 'a:c:v'
            )
        )
        # Test wildcard match
        self.assertTrue(
            salt.utils.data.subdict_match(
                test_three_level_dict, 'a:*:c:v'
            )
        )

    def test_subdict_match_with_wildcards(self):
        '''
        Tests subdict matching when wildcards are used in the expression
        '''
        data = {
            'a': {
                'b': {
                    'ç': 'd',
                    'é': ['eff', 'gee', '8ch'],
                    'ĩ': {'j': 'k'}
                }
            }
        }
        assert salt.utils.data.subdict_match(data, '*:*:*:*')
        assert salt.utils.data.subdict_match(data, 'a:*:*:*')
        assert salt.utils.data.subdict_match(data, 'a:b:*:*')
        assert salt.utils.data.subdict_match(data, 'a:b:ç:*')
        assert salt.utils.data.subdict_match(data, 'a:b:*:d')
        assert salt.utils.data.subdict_match(data, 'a:*:ç:d')
        assert salt.utils.data.subdict_match(data, '*:b:ç:d')
        assert salt.utils.data.subdict_match(data, '*:*:ç:d')
        assert salt.utils.data.subdict_match(data, '*:*:*:d')
        assert salt.utils.data.subdict_match(data, 'a:*:*:d')
        assert salt.utils.data.subdict_match(data, 'a:b:*:ef*')
        assert salt.utils.data.subdict_match(data, 'a:b:*:g*')
        assert salt.utils.data.subdict_match(data, 'a:b:*:j:*')
        assert salt.utils.data.subdict_match(data, 'a:b:*:j:k')
        assert salt.utils.data.subdict_match(data, 'a:b:*:*:k')
        assert salt.utils.data.subdict_match(data, 'a:b:*:*:*')

    def test_traverse_dict(self):
        test_two_level_dict = {'foo': {'bar': 'baz'}}

        self.assertDictEqual(
            {'not_found': 'nope'},
            salt.utils.data.traverse_dict(
                test_two_level_dict, 'foo:bar:baz', {'not_found': 'nope'}
            )
        )
        self.assertEqual(
            'baz',
            salt.utils.data.traverse_dict(
                test_two_level_dict, 'foo:bar', {'not_found': 'not_found'}
            )
        )

    def test_traverse_dict_and_list(self):
        test_two_level_dict = {'foo': {'bar': 'baz'}}
        test_two_level_dict_and_list = {
            'foo': ['bar', 'baz', {'lorem': {'ipsum': [{'dolor': 'sit'}]}}]
        }

        # Check traversing too far: salt.utils.data.traverse_dict_and_list() returns
        # the value corresponding to a given key path, and baz is a value
        # corresponding to the key path foo:bar.
        self.assertDictEqual(
            {'not_found': 'nope'},
            salt.utils.data.traverse_dict_and_list(
                test_two_level_dict, 'foo:bar:baz', {'not_found': 'nope'}
            )
        )
        # Now check to ensure that foo:bar corresponds to baz
        self.assertEqual(
            'baz',
            salt.utils.data.traverse_dict_and_list(
                test_two_level_dict, 'foo:bar', {'not_found': 'not_found'}
            )
        )
        # Check traversing too far
        self.assertDictEqual(
            {'not_found': 'nope'},
            salt.utils.data.traverse_dict_and_list(
                test_two_level_dict_and_list, 'foo:bar', {'not_found': 'nope'}
            )
        )
        # Check index 1 (2nd element) of list corresponding to path 'foo'
        self.assertEqual(
            'baz',
            salt.utils.data.traverse_dict_and_list(
                test_two_level_dict_and_list, 'foo:1', {'not_found': 'not_found'}
            )
        )
        # Traverse a couple times into dicts embedded in lists
        self.assertEqual(
            'sit',
            salt.utils.data.traverse_dict_and_list(
                test_two_level_dict_and_list,
                'foo:lorem:ipsum:dolor',
                {'not_found': 'not_found'}
            )
        )

    def test_compare_dicts(self):
        ret = salt.utils.data.compare_dicts(old={'foo': 'bar'}, new={'foo': 'bar'})
        self.assertEqual(ret, {})

        ret = salt.utils.data.compare_dicts(old={'foo': 'bar'}, new={'foo': 'woz'})
        expected_ret = {'foo': {'new': 'woz', 'old': 'bar'}}
        self.assertDictEqual(ret, expected_ret)

    def test_decode(self):
        '''
        Companion to test_decode_to_str, they should both be kept up-to-date
        with one another.

        NOTE: This uses the lambda "_b" defined above in the global scope,
        which encodes a string to a bytestring, assuming utf-8.
        '''
        expected = [
            'unicode_str',
            'питон',
            123,
            456.789,
            True,
            False,
            None,
            'яйца',
            BYTES,
            [123, 456.789, 'спам', True, False, None, 'яйца', BYTES],
            (987, 654.321, 'яйца', 'яйца', None, (True, 'яйца', BYTES)),
            {'str_key': 'str_val',
             None: True,
             123: 456.789,
             'яйца': BYTES,
             'subdict': {'unicode_key': 'яйца',
                         'tuple': (123, 'hello', 'world', True, 'яйца', BYTES),
                         'list': [456, 'спам', False, 'яйца', BYTES]}},
            OrderedDict([('foo', 'bar'), (123, 456), ('яйца', BYTES)])
        ]

        ret = salt.utils.data.decode(
            self.test_data,
            keep=True,
            normalize=True,
            preserve_dict_class=True,
            preserve_tuples=True)
        self.assertEqual(ret, expected)

        # The binary data in the data structure should fail to decode, even
        # using the fallback, and raise an exception.
        self.assertRaises(
            UnicodeDecodeError,
            salt.utils.data.decode,
            self.test_data,
            keep=False,
            normalize=True,
            preserve_dict_class=True,
            preserve_tuples=True)

        # Now munge the expected data so that we get what we would expect if we
        # disable preservation of dict class and tuples
        expected[10] = [987, 654.321, 'яйца', 'яйца', None, [True, 'яйца', BYTES]]
        expected[11]['subdict']['tuple'] = [123, 'hello', 'world', True, 'яйца', BYTES]
        expected[12] = {'foo': 'bar', 123: 456, 'яйца': BYTES}

        ret = salt.utils.data.decode(
            self.test_data,
            keep=True,
            normalize=True,
            preserve_dict_class=False,
            preserve_tuples=False)
        self.assertEqual(ret, expected)

        # Now test single non-string, non-data-structure items, these should
        # return the same value when passed to this function
        for item in (123, 4.56, True, False, None):
            log.debug('Testing decode of %s', item)
            self.assertEqual(salt.utils.data.decode(item), item)

        # Test single strings (not in a data structure)
        self.assertEqual(salt.utils.data.decode('foo'), 'foo')
        self.assertEqual(salt.utils.data.decode(_b('bar')), 'bar')
        self.assertEqual(salt.utils.data.decode(EGGS, normalize=True), 'яйца')
        self.assertEqual(salt.utils.data.decode(EGGS, normalize=False), EGGS)

        # Test binary blob
        self.assertEqual(salt.utils.data.decode(BYTES, keep=True), BYTES)
        self.assertRaises(
            UnicodeDecodeError,
            salt.utils.data.decode,
            BYTES,
            keep=False)

    def test_decode_to_str(self):
        '''
        Companion to test_decode, they should both be kept up-to-date with one
        another.

        NOTE: This uses the lambda "_s" defined above in the global scope,
        which converts the string/bytestring to a str type.
        '''
        expected = [
            _s('unicode_str'),
            _s('питон'),
            123,
            456.789,
            True,
            False,
            None,
            _s('яйца'),
            BYTES,
            [123, 456.789, _s('спам'), True, False, None, _s('яйца'), BYTES],
            (987, 654.321, _s('яйца'), _s('яйца'), None, (True, _s('яйца'), BYTES)),
            {_s('str_key'): _s('str_val'),
             None: True,
             123: 456.789,
             _s('яйца'): BYTES,
             _s('subdict'): {
                 _s('unicode_key'): _s('яйца'),
                 _s('tuple'): (123, _s('hello'), _s('world'), True, _s('яйца'), BYTES),
                 _s('list'): [456, _s('спам'), False, _s('яйца'), BYTES]}},
            OrderedDict([(_s('foo'), _s('bar')), (123, 456), (_s('яйца'), BYTES)])
        ]

        ret = salt.utils.data.decode(
            self.test_data,
            keep=True,
            normalize=True,
            preserve_dict_class=True,
            preserve_tuples=True,
            to_str=True)
        self.assertEqual(ret, expected)

        if six.PY3:
            # The binary data in the data structure should fail to decode, even
            # using the fallback, and raise an exception.
            self.assertRaises(
                UnicodeDecodeError,
                salt.utils.data.decode,
                self.test_data,
                keep=False,
                normalize=True,
                preserve_dict_class=True,
                preserve_tuples=True,
                to_str=True)

        # Now munge the expected data so that we get what we would expect if we
        # disable preservation of dict class and tuples
        expected[10] = [987, 654.321, _s('яйца'), _s('яйца'), None, [True, _s('яйца'), BYTES]]
        expected[11][_s('subdict')][_s('tuple')] = [123, _s('hello'), _s('world'), True, _s('яйца'), BYTES]
        expected[12] = {_s('foo'): _s('bar'), 123: 456, _s('яйца'): BYTES}

        ret = salt.utils.data.decode(
            self.test_data,
            keep=True,
            normalize=True,
            preserve_dict_class=False,
            preserve_tuples=False,
            to_str=True)
        self.assertEqual(ret, expected)

        # Now test single non-string, non-data-structure items, these should
        # return the same value when passed to this function
        for item in (123, 4.56, True, False, None):
            log.debug('Testing decode of %s', item)
            self.assertEqual(salt.utils.data.decode(item, to_str=True), item)

        # Test single strings (not in a data structure)
        self.assertEqual(salt.utils.data.decode('foo', to_str=True), _s('foo'))
        self.assertEqual(salt.utils.data.decode(_b('bar'), to_str=True), _s('bar'))

        # Test binary blob
        self.assertEqual(
            salt.utils.data.decode(BYTES, keep=True, to_str=True),
            BYTES
        )
        if six.PY3:
            self.assertRaises(
                UnicodeDecodeError,
                salt.utils.data.decode,
                BYTES,
                keep=False,
                to_str=True)

    @skipIf(NO_MOCK, NO_MOCK_REASON)
    def test_decode_fallback(self):
        '''
        Test fallback to utf-8
        '''
        with patch.object(builtins, '__salt_system_encoding__', 'ascii'):
            self.assertEqual(salt.utils.data.decode(_b('яйца')), 'яйца')

    def test_encode(self):
        '''
        NOTE: This uses the lambda "_b" defined above in the global scope,
        which encodes a string to a bytestring, assuming utf-8.
        '''
        expected = [
            _b('unicode_str'),
            _b('питон'),
            123,
            456.789,
            True,
            False,
            None,
            _b(EGGS),
            BYTES,
            [123, 456.789, _b('спам'), True, False, None, _b(EGGS), BYTES],
            (987, 654.321, _b('яйца'), _b(EGGS), None, (True, _b(EGGS), BYTES)),
            {_b('str_key'): _b('str_val'),
             None: True,
             123: 456.789,
             _b(EGGS): BYTES,
             _b('subdict'): {_b('unicode_key'): _b(EGGS),
                             _b('tuple'): (123, _b('hello'), _b('world'), True, _b(EGGS), BYTES),
                             _b('list'): [456, _b('спам'), False, _b(EGGS), BYTES]}},
             OrderedDict([(_b('foo'), _b('bar')), (123, 456), (_b(EGGS), BYTES)])
        ]

        # Both keep=True and keep=False should work because the BYTES data is
        # already bytes.
        ret = salt.utils.data.encode(
            self.test_data,
            keep=True,
            preserve_dict_class=True,
            preserve_tuples=True)
        self.assertEqual(ret, expected)
        ret = salt.utils.data.encode(
            self.test_data,
            keep=False,
            preserve_dict_class=True,
            preserve_tuples=True)
        self.assertEqual(ret, expected)

        # Now munge the expected data so that we get what we would expect if we
        # disable preservation of dict class and tuples
        expected[10] = [987, 654.321, _b('яйца'), _b(EGGS), None, [True, _b(EGGS), BYTES]]
        expected[11][_b('subdict')][_b('tuple')] = [
            123, _b('hello'), _b('world'), True, _b(EGGS), BYTES
        ]
        expected[12] = {_b('foo'): _b('bar'), 123: 456, _b(EGGS): BYTES}

        ret = salt.utils.data.encode(
            self.test_data,
            keep=True,
            preserve_dict_class=False,
            preserve_tuples=False)
        self.assertEqual(ret, expected)
        ret = salt.utils.data.encode(
            self.test_data,
            keep=False,
            preserve_dict_class=False,
            preserve_tuples=False)
        self.assertEqual(ret, expected)

        # Now test single non-string, non-data-structure items, these should
        # return the same value when passed to this function
        for item in (123, 4.56, True, False, None):
            log.debug('Testing encode of %s', item)
            self.assertEqual(salt.utils.data.encode(item), item)

        # Test single strings (not in a data structure)
        self.assertEqual(salt.utils.data.encode('foo'), _b('foo'))
        self.assertEqual(salt.utils.data.encode(_b('bar')), _b('bar'))

        # Test binary blob, nothing should happen even when keep=False since
        # the data is already bytes
        self.assertEqual(salt.utils.data.encode(BYTES, keep=True), BYTES)
        self.assertEqual(salt.utils.data.encode(BYTES, keep=False), BYTES)

    def test_encode_keep(self):
        '''
        Whereas we tested the keep argument in test_decode, it is much easier
        to do a more comprehensive test of keep in its own function where we
        can force the encoding.
        '''
        unicode_str = 'питон'
        encoding = 'ascii'

        # Test single string
        self.assertEqual(
            salt.utils.data.encode(unicode_str, encoding, keep=True),
            unicode_str)
        self.assertRaises(
            UnicodeEncodeError,
            salt.utils.data.encode,
            unicode_str,
            encoding,
            keep=False)

        data = [
            unicode_str,
            [b'foo', [unicode_str], {b'key': unicode_str}, (unicode_str,)],
            {b'list': [b'foo', unicode_str],
             b'dict': {b'key': unicode_str},
             b'tuple': (b'foo', unicode_str)},
            ([b'foo', unicode_str], {b'key': unicode_str}, (unicode_str,))
        ]

        # Since everything was a bytestring aside from the bogus data, the
        # return data should be identical. We don't need to test recursive
        # decoding, that has already been tested in test_encode.
        self.assertEqual(
            salt.utils.data.encode(data, encoding,
                                   keep=True, preserve_tuples=True),
            data
        )
        self.assertRaises(
            UnicodeEncodeError,
            salt.utils.data.encode,
            data,
            encoding,
            keep=False,
            preserve_tuples=True)

        for index, item in enumerate(data):
            self.assertEqual(
                salt.utils.data.encode(data[index], encoding,
                                       keep=True, preserve_tuples=True),
                data[index]
            )
            self.assertRaises(
                UnicodeEncodeError,
                salt.utils.data.encode,
                data[index],
                encoding,
                keep=False,
                preserve_tuples=True)

    @skipIf(NO_MOCK, NO_MOCK_REASON)
    def test_encode_fallback(self):
        '''
        Test fallback to utf-8
        '''
        with patch.object(builtins, '__salt_system_encoding__', 'ascii'):
            self.assertEqual(salt.utils.data.encode('яйца'), _b('яйца'))
        with patch.object(builtins, '__salt_system_encoding__', 'CP1252'):
            self.assertEqual(salt.utils.data.encode('Ψ'), _b('Ψ'))

    def test_repack_dict(self):
        list_of_one_element_dicts = [{'dict_key_1': 'dict_val_1'},
                                     {'dict_key_2': 'dict_val_2'},
                                     {'dict_key_3': 'dict_val_3'}]
        expected_ret = {'dict_key_1': 'dict_val_1',
                        'dict_key_2': 'dict_val_2',
                        'dict_key_3': 'dict_val_3'}
        ret = salt.utils.data.repack_dictlist(list_of_one_element_dicts)
        self.assertDictEqual(ret, expected_ret)

        # Try with yaml
        yaml_key_val_pair = '- key1: val1'
        ret = salt.utils.data.repack_dictlist(yaml_key_val_pair)
        self.assertDictEqual(ret, {'key1': 'val1'})

        # Make sure we handle non-yaml junk data
        ret = salt.utils.data.repack_dictlist(LOREM_IPSUM)
        self.assertDictEqual(ret, {})

    def test_stringify(self):
        self.assertRaises(TypeError, salt.utils.data.stringify, 9)
        self.assertEqual(
            salt.utils.data.stringify(['one', 'two', str('three'), 4, 5]),  # future lint: disable=blacklisted-function
            ['one', 'two', 'three', '4', '5']
        )



# coding: utf-8

# Python libs
from __future__ import absolute_import

# Salt libs
import salt.beacons.glxinfo as glxinfo

# Salt testing libs
from tests.support.unit import skipIf, TestCase
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, patch, Mock

# Import test suite libs
from tests.support.mixins import LoaderModuleMockMixin


@skipIf(NO_MOCK, NO_MOCK_REASON)
class GLXInfoBeaconTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test case for salt.beacons.glxinfo
    '''

    def setup_loader_modules(self):
        return {glxinfo: {'last_state': {}}}

    def test_no_glxinfo_command(self):
        with patch('salt.utils.path.which') as mock:
            mock.return_value = None

            ret = glxinfo.__virtual__()

            mock.assert_called_once_with('glxinfo')
            self.assertFalse(ret)

    def test_with_glxinfo_command(self):
        with patch('salt.utils.path.which') as mock:
            mock.return_value = '/usr/bin/glxinfo'

            ret = glxinfo.__virtual__()

            mock.assert_called_once_with('glxinfo')
            self.assertEqual(ret, 'glxinfo')

    def test_non_list_config(self):
        config = {}

        ret = glxinfo.validate(config)

        self.assertEqual(ret, (False, 'Configuration for glxinfo '
                                      'beacon must be a list.'))

    def test_no_user(self):
        config = [{'screen_event': True}]

        _expected = (False, 'Configuration for glxinfo beacon must '
                            'include a user as glxinfo is not '
                            'available to root.')
        ret = glxinfo.validate(config)
        self.assertEqual(ret, _expected)

    def test_screen_state(self):
        config = [{'screen_event': True,
                  'user': 'frank'}]

        mock = Mock(return_value=0)
        with patch.dict(glxinfo.__salt__, {'cmd.retcode': mock}):

            ret = glxinfo.validate(config)
            self.assertEqual(ret, (True, 'Valid beacon configuration'))

            ret = glxinfo.beacon(config)
            self.assertEqual(ret, [{'tag': 'screen_event',
                                    'screen_available': True}])
            mock.assert_called_once_with('DISPLAY=:0 glxinfo',
                                         runas='frank', python_shell=True)

    def test_screen_state_missing(self):
        config = [{'screen_event': True, 'user': 'frank'}]

        mock = Mock(return_value=255)
        with patch.dict(glxinfo.__salt__, {'cmd.retcode': mock}):
            ret = glxinfo.validate(config)
            self.assertEqual(ret, (True, 'Valid beacon configuration'))

            ret = glxinfo.beacon(config)
            self.assertEqual(ret, [{'tag': 'screen_event',
                                    'screen_available': False}])

    def test_screen_state_no_repeat(self):
        config = [{'screen_event': True, 'user': 'frank'}]

        mock = Mock(return_value=255)
        with patch.dict(glxinfo.__salt__, {'cmd.retcode': mock}):
            ret = glxinfo.validate(config)
            self.assertEqual(ret, (True, 'Valid beacon configuration'))

            ret = glxinfo.beacon(config)
            self.assertEqual(ret, [{'tag': 'screen_event',
                                    'screen_available': False}])

            ret = glxinfo.beacon(config)
            self.assertEqual(ret, [])

    def test_screen_state_change(self):
        config = [{'screen_event': True, 'user': 'frank'}]

        mock = Mock(side_effect=[255, 0])
        with patch.dict(glxinfo.__salt__, {'cmd.retcode': mock}):
            ret = glxinfo.validate(config)
            self.assertEqual(ret, (True, 'Valid beacon configuration'))

            ret = glxinfo.beacon(config)
            self.assertEqual(ret, [{'tag': 'screen_event',
                                    'screen_available': False}])

            ret = glxinfo.beacon(config)
            self.assertEqual(ret, [{'tag': 'screen_event',
                                    'screen_available': True}])



# -*- coding: utf-8 -*-
'''
    :codeauthor: Pedro Algarvio (pedro@algarvio.me)
    :copyright: Copyright 2017 by the SaltStack Team, see AUTHORS for more details.
    :license: Apache 2.0, see LICENSE for more details.


    tests.support.paths
    ~~~~~~~~~~~~~~~~~~~

    Tests related paths
'''

# Import python libs
from __future__ import absolute_import
import os
import re
import sys
import stat
import logging
import tempfile

import salt.utils.path

log = logging.getLogger(__name__)

TESTS_DIR = os.path.dirname(os.path.dirname(os.path.normpath(os.path.abspath(__file__))))
if TESTS_DIR.startswith('//'):
    # Have we been given an initial double forward slash? Ditch it!
    TESTS_DIR = TESTS_DIR[1:]
if sys.platform.startswith('win'):
    TESTS_DIR = os.path.normcase(TESTS_DIR)
CODE_DIR = os.path.dirname(TESTS_DIR)
if sys.platform.startswith('win'):
    CODE_DIR = CODE_DIR.replace('\\', '\\\\')
UNIT_TEST_DIR = os.path.join(TESTS_DIR, 'unit')
INTEGRATION_TEST_DIR = os.path.join(TESTS_DIR, 'integration')

# Let's inject CODE_DIR so salt is importable if not there already
if TESTS_DIR in sys.path:
    sys.path.remove(TESTS_DIR)
if CODE_DIR in sys.path and sys.path[0] != CODE_DIR:
    sys.path.remove(CODE_DIR)
if CODE_DIR not in sys.path:
    sys.path.insert(0, CODE_DIR)
if TESTS_DIR not in sys.path:
    sys.path.insert(1, TESTS_DIR)

SYS_TMP_DIR = os.path.abspath(os.path.realpath(
    # Avoid ${TMPDIR} and gettempdir() on MacOS as they yield a base path too long
    # for unix sockets: ``error: AF_UNIX path too long``
    # Gentoo Portage prefers ebuild tests are rooted in ${TMPDIR}
    os.environ.get('TMPDIR', tempfile.gettempdir()) if not sys.platform.startswith('darwin') else '/tmp'
))
TMP = os.path.join(SYS_TMP_DIR, 'salt-tests-tmpdir')
FILES = os.path.join(INTEGRATION_TEST_DIR, 'files')
BASE_FILES = os.path.join(FILES, 'file', 'base')
PROD_FILES = os.path.join(FILES, 'file', 'prod')
PYEXEC = 'python{0}.{1}'.format(*sys.version_info)
MOCKBIN = os.path.join(INTEGRATION_TEST_DIR, 'mockbin')
SCRIPT_DIR = os.path.join(CODE_DIR, 'scripts')
TMP_STATE_TREE = os.path.join(SYS_TMP_DIR, 'salt-temp-state-tree')
TMP_PILLAR_TREE = os.path.join(SYS_TMP_DIR, 'salt-temp-pillar-tree')
TMP_PRODENV_STATE_TREE = os.path.join(SYS_TMP_DIR, 'salt-temp-prodenv-state-tree')
TMP_CONF_DIR = os.path.join(TMP, 'config')
TMP_SUB_MINION_CONF_DIR = os.path.join(TMP_CONF_DIR, 'sub-minion')
TMP_SYNDIC_MINION_CONF_DIR = os.path.join(TMP_CONF_DIR, 'syndic-minion')
TMP_SYNDIC_MASTER_CONF_DIR = os.path.join(TMP_CONF_DIR, 'syndic-master')
TMP_PROXY_CONF_DIR = os.path.join(TMP_CONF_DIR, 'proxy')
CONF_DIR = os.path.join(INTEGRATION_TEST_DIR, 'files', 'conf')
PILLAR_DIR = os.path.join(FILES, 'pillar')
TMP_SCRIPT_DIR = os.path.join(TMP, 'scripts')
ENGINES_DIR = os.path.join(FILES, 'engines')
LOG_HANDLERS_DIR = os.path.join(FILES, 'log_handlers')


SCRIPT_TEMPLATES = {
    'salt': [
        'from salt.scripts import salt_main\n',
        'if __name__ == \'__main__\':\n'
        '    salt_main()'
    ],
    'salt-api': [
        'import salt.cli\n',
        'def main():\n',
        '    sapi = salt.cli.SaltAPI()',
        '    sapi.start()\n',
        'if __name__ == \'__main__\':',
        '    main()'
    ],
    'common': [
        'from salt.scripts import salt_{0}\n',
        'import salt.utils.platform\n\n',
        'if __name__ == \'__main__\':\n',
        '    if salt.utils.platform.is_windows():\n',
        '        import os.path\n',
        '        import py_compile\n',
        '        cfile = os.path.splitext(__file__)[0] + ".pyc"\n',
        '        if not os.path.exists(cfile):\n',
        '            py_compile.compile(__file__, cfile)\n',
        '    salt_{0}()'
    ]
}


def test_mods():
    '''
    A generator which returns all of the test files
    '''
    test_re = re.compile(r'^test_.+\.py$')
    for dirname in (UNIT_TEST_DIR, INTEGRATION_TEST_DIR):
        test_type = os.path.basename(dirname)
        for root, _, files in salt.utils.path.os_walk(dirname):
            parent_mod = root[len(dirname):].lstrip(os.sep).replace(os.sep, '.')
            for filename in files:
                if test_re.match(filename):
                    mod_name = test_type
                    if parent_mod:
                        mod_name += '.' + parent_mod
                    mod_name += '.' + filename[:-3]
                    yield mod_name


class ScriptPathMixin(object):

    def get_script_path(self, script_name):
        '''
        Return the path to a testing runtime script
        '''
        if not os.path.isdir(TMP_SCRIPT_DIR):
            os.makedirs(TMP_SCRIPT_DIR)

        script_path = os.path.join(TMP_SCRIPT_DIR,
                                   'cli_{0}.py'.format(script_name.replace('-', '_')))

        if not os.path.isfile(script_path):
            log.info('Generating {0}'.format(script_path))

            # Late import
            import salt.utils.files

            with salt.utils.files.fopen(script_path, 'w') as sfh:
                script_template = SCRIPT_TEMPLATES.get(script_name, None)
                if script_template is None:
                    script_template = SCRIPT_TEMPLATES.get('common', None)
                if script_template is None:
                    raise RuntimeError(
                        '{0} does not know how to handle the {1} script'.format(
                            self.__class__.__name__,
                            script_name
                        )
                    )
                sfh.write(
                    '#!{0}\n\n'.format(sys.executable) +
                    'import sys\n' +
                    'CODE_DIR = r"{0}"\n'.format(CODE_DIR) +
                    'if CODE_DIR not in sys.path:\n' +
                    '    sys.path.insert(0, CODE_DIR)\n\n' +
                    '\n'.join(script_template).format(script_name.replace('salt-', ''))
                )
            fst = os.stat(script_path)
            os.chmod(script_path, fst.st_mode | stat.S_IEXEC)

        log.info('Returning script path %r', script_path)
        return script_path



# -*- coding: utf-8 -*-
'''
Base classes for napalm unit tests

:codeauthor: :email:`Anthony Shaw <anthonyshaw@apache.org>`
'''

from __future__ import absolute_import
from functools import wraps


TEST_INTERFACES = {
    'Management1': {
        'is_up': False,
        'is_enabled': False,
        'description': u'',
        'last_flapped': -1,
        'speed': 1000,
        'mac_address': u'dead:beef:dead',
    }
}

# Test data
TEST_FACTS = {
    '__opts__': {},
    'OPTIONAL_ARGS': {},
    'uptime': 'Forever',
    'UP': True,
    'HOSTNAME': 'test-device.com',
    'hostname': 'test-device.com',
    'username': 'admin',
    'os_version': '1.2.3',
    'model': 'test_model',
    'serial_number': '123456',
    'vendor': 'cisco',
    'interface_list': TEST_INTERFACES
}

TEST_ENVIRONMENT = {
    'hot': 'yes'
}

TEST_COMMAND_RESPONSE = {
    'show run': 'all the command output'
}

TEST_TRACEROUTE_RESPONSE = {
    'success': {
        1: {
            'probes': {
                1: {
                    'rtt': 1.123,
                    'ip_address': u'206.223.116.21',
                    'host_name': u'eqixsj-google-gige.google.com'
                }
            }
        }
    }
}

TEST_PING_RESPONSE = {
    'success': {
        'probes_sent': 5,
        'packet_loss': 0,
        'rtt_min': 72.158,
        'rtt_max': 72.433,
        'rtt_avg': 72.268,
        'rtt_stddev': 0.094,
        'results': [
            {
                'ip_address': '1.1.1.1',
                'rtt': 72.248
            }
        ]
    }
}

TEST_ARP_TABLE = [
    {
        'interface': 'MgmtEth0/RSP0/CPU0/0',
        'mac': '5C:5E:AB:DA:3C:F0',
        'ip': '172.17.17.1',
        'age': 1454496274.84
    }
]

TEST_IPADDRS = {
    'FastEthernet8': {
        'ipv4': {
            '10.66.43.169': {
                'prefix_length': 22
            }
        }
    }
}

TEST_INTERFACES = {
    'Management1': {
        'is_up': False,
        'is_enabled': False,
        'description': u'',
        'last_flapped': -1,
        'speed': 1000,
        'mac_address': u'dead:beef:dead',
    }
}

TEST_LLDP_NEIGHBORS = {
    u'Ethernet2':
        [
            {
                'hostname': u'junos-unittest',
                'port': u'520',
            }
        ]
}

TEST_MAC_TABLE = [
    {
        'mac': '00:1C:58:29:4A:71',
        'interface': 'Ethernet47',
        'vlan': 100,
        'static': False,
        'active': True,
        'moves': 1,
        'last_move': 1454417742.58
    }
]

TEST_RUNNING_CONFIG = {
    'one': 'two'
}

TEST_OPTICS = {
    'et1': {
        'physical_channels': {
            'channel': [
                {
                    'index': 0,
                    'state': {
                        'input_power': {
                            'instant': 0.0,
                            'avg': 0.0,
                            'min': 0.0,
                            'max': 0.0,
                        },
                        'output_power': {
                            'instant': 0.0,
                            'avg': 0.0,
                            'min': 0.0,
                            'max': 0.0,
                        },
                        'laser_bias_current': {
                            'instant': 0.0,
                            'avg': 0.0,
                            'min': 0.0,
                            'max': 0.0,
                        },
                    }
                }
            ]
        }
    }
}

TEST_BGP_CONFIG = {
    'test': 'value'
}

TEST_BGP_NEIGHBORS = {
    'default': {
        8121: [
            {
                'up': True,
                'local_as': 13335,
                'remote_as': 8121,
                'local_address': u'172.101.76.1',
                'local_address_configured': True,
                'local_port': 179,
                'remote_address': u'192.247.78.0',
                'router_id': u'192.168.0.1',
                'remote_port': 58380,
                'multihop': False,
                'import_policy': u'4-NTT-TRANSIT-IN',
                'export_policy': u'4-NTT-TRANSIT-OUT',
                'input_messages': 123,
                'output_messages': 13,
                'input_updates': 123,
                'output_updates': 5,
                'messages_queued_out': 23,
                'connection_state': u'Established',
                'previous_connection_state': u'EstabSync',
                'last_event': u'RecvKeepAlive',
                'suppress_4byte_as': False,
                'local_as_prepend': False,
                'holdtime': 90,
                'configured_holdtime': 90,
                'keepalive': 30,
                'configured_keepalive': 30,
                'active_prefix_count': 132808,
                'received_prefix_count': 566739,
                'accepted_prefix_count': 566479,
                'suppressed_prefix_count': 0,
                'advertise_prefix_count': 0,
                'flap_count': 27
            }
        ]
    }
}

TEST_TERM_CONFIG = {
    'result': True,
    'already_configured': False
}

TEST_NTP_PEERS = {
    '192.168.0.1': 1,
    '172.17.17.1': 2,
    '172.17.17.2': 3,
    '2400:cb00:6:1024::c71b:840a': 4
}

TEST_NTP_SERVERS = {
    '192.168.0.1': 1,
    '172.17.17.1': 2,
    '172.17.17.2': 3,
    '2400:cb00:6:1024::c71b:840a': 4
}

TEST_NTP_STATS = [
    {
        'remote': u'188.114.101.4',
        'referenceid': u'188.114.100.1',
        'synchronized': True,
        'stratum': 4,
        'type': u'-',
        'when': u'107',
        'hostpoll': 256,
        'reachability': 377,
        'delay': 164.228,
        'offset': -13.866,
        'jitter': 2.695
    }
]

TEST_PROBES_CONFIG = {
    'probe1': {
        'test1': {
            'probe_type': 'icmp-ping',
            'target': '192.168.0.1',
            'source': '192.168.0.2',
            'probe_count': 13,
            'test_interval': 3
        },
        'test2': {
            'probe_type': 'http-ping',
            'target': '172.17.17.1',
            'source': '192.17.17.2',
            'probe_count': 5,
            'test_interval': 60
        }
    }
}

TEST_PROBES_RESULTS = {
    'probe1':  {
        'test1': {
            'last_test_min_delay': 63.120,
            'global_test_min_delay': 62.912,
            'current_test_avg_delay': 63.190,
            'global_test_max_delay': 177.349,
            'current_test_max_delay': 63.302,
            'global_test_avg_delay': 63.802,
            'last_test_avg_delay': 63.438,
            'last_test_max_delay': 65.356,
            'probe_type': 'icmp-ping',
            'rtt': 63.138,
            'last_test_loss': 0,
            'round_trip_jitter': -59.0,
            'target': '192.168.0.1',
            'source': '192.168.0.2',
            'probe_count': 15,
            'current_test_min_delay': 63.138
        },
        'test2': {
            'last_test_min_delay': 176.384,
            'global_test_min_delay': 169.226,
            'current_test_avg_delay': 177.098,
            'global_test_max_delay': 292.628,
            'current_test_max_delay': 180.055,
            'global_test_avg_delay': 177.959,
            'last_test_avg_delay': 177.178,
            'last_test_max_delay': 184.671,
            'probe_type': 'icmp-ping',
            'rtt': 176.449,
            'last_test_loss': 0,
            'round_trip_jitter': -34.0,
            'target': '172.17.17.1',
            'source': '172.17.17.2',
            'probe_count': 15,
            'current_test_min_delay': 176.402
        }
    }
}

TEST_ROUTE = {
    '172.16.0.0/25': [
        {
            'protocol': 'BGP',
            'last_active': True,
            'current_active': True,
            'age': 1178693,
            'routing_table': 'inet.0',
            'next_hop': '192.168.0.11',
            'outgoing_interface': 'xe-1/1/1.100',
            'preference': 170,
            'selected_next_hop': False,
            'protocol_attributes': {
                'remote_as': 65001,
                'metric': 5,
                'local_as': 13335,
                'as_path': '',
                'remote_address': '192.168.0.11',
                'metric2': 0,
                'local_preference': 0,
                'communities': [
                    '0:2',
                    'no-export'
                ],
                'preference2': -1
            },
            'inactive_reason': ''
        },
        {
            'protocol': 'BGP',
            'last_active': False,
            'current_active': False,
            'age': 2359429,
            'routing_table': 'inet.0',
            'next_hop': '192.168.0.17',
            'outgoing_interface': 'xe-1/1/1.100',
            'preference': 170,
            'selected_next_hop': True,
            'protocol_attributes': {
                'remote_as': 65001,
                'metric': 5,
                'local_as': 13335,
                'as_path': '',
                'remote_address': '192.168.0.17',
                'metric2': 0,
                'local_preference': 0,
                'communities': [
                    '0:3',
                    'no-export'
                ],
                'preference2': -1
            },
            'inactive_reason': 'Not Best in its group - Router ID'
        }
    ]
}

TEST_SNMP_INFO = {
    'test_': 'value'
}

TEST_USERS = {
    'mircea': {
        'level': 15,
        'password': '$1$0P70xKPa$4jt5/10cBTckk6I/w/',
        'sshkeys': [
            'ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAABAQC4pFn+shPwTb2yELO4L7NtQrKOJXNeCl1je\
            l9STXVaGnRAnuc2PXl35vnWmcUq6YbUEcgUTRzzXfmelJKuVJTJIlMXii7h2xkbQp0YZIEs4P\
            8ipwnRBAxFfk/ZcDsdfsdfsdfsdN56ejk345jhk345jk345jk341p3A/9LIL7l6YewLBCwJj6\
            D+fWSJ0/YW+7oH17Fk2HH+tw0L5PcWLHkwA4t60iXn16qDbIk/ze6jv2hDGdCdz7oYQeCE55C\
            CHOHMJWYfN3jcL4s0qv8/u6Ka1FVkV7iMmro7ChThoV/5snI4Ljf2wKqgHH7TfNaCfpU0WvHA\
            nTs8zhOrGScSrtb mircea@master-roshi'
        ]
    }
}


class MockNapalmDevice(object):
    '''Setup a mock device for our tests'''
    def get_facts(self):
        return TEST_FACTS

    def get_environment(self):
        return TEST_ENVIRONMENT

    def get_arp_table(self):
        return TEST_ARP_TABLE

    def get(self, key, default=None, *args, **kwargs):
        try:
            if key == 'DRIVER':
                return self
            return TEST_FACTS[key]
        except KeyError:
            return default

    def cli(self, commands, *args, **kwargs):
        assert commands[0] == 'show run'
        return TEST_COMMAND_RESPONSE

    def traceroute(self, destination, **kwargs):
        assert destination == 'destination.com'
        return TEST_TRACEROUTE_RESPONSE

    def ping(self, destination, **kwargs):
        assert destination == 'destination.com'
        return TEST_PING_RESPONSE

    def get_config(self, retrieve='all'):
        assert retrieve == 'running'
        return TEST_RUNNING_CONFIG

    def get_interfaces_ip(self, **kwargs):
        return TEST_IPADDRS

    def get_interfaces(self, **kwargs):
        return TEST_INTERFACES

    def get_lldp_neighbors_detail(self, **kwargs):
        return TEST_LLDP_NEIGHBORS

    def get_mac_address_table(self, **kwargs):
        return TEST_MAC_TABLE

    def get_optics(self, **kwargs):
        return TEST_OPTICS

    def load_merge_candidate(self, filename=None, config=None):
        assert config == 'new config'
        return TEST_RUNNING_CONFIG

    def load_replace_candidate(self, filename=None, config=None):
        assert config == 'new config'
        return TEST_RUNNING_CONFIG

    def commit_config(self, **kwargs):
        return TEST_RUNNING_CONFIG

    def discard_config(self, **kwargs):
        return TEST_RUNNING_CONFIG

    def compare_config(self, **kwargs):
        return TEST_RUNNING_CONFIG

    def rollback(self, **kwargs):
        return TEST_RUNNING_CONFIG

    def get_bgp_config(self, **kwargs):
        return TEST_BGP_CONFIG

    def get_bgp_neighbors_detail(self, neighbor_address=None, **kwargs):
        assert neighbor_address is None or "test_address"
        return TEST_BGP_NEIGHBORS

    def get_ntp_peers(self, **kwargs):
        return TEST_NTP_PEERS

    def get_ntp_servers(self, **kwargs):
        return TEST_NTP_SERVERS

    def get_ntp_stats(self, **kwargs):
        return TEST_NTP_STATS

    def get_probes_config(self, **kwargs):
        return TEST_PROBES_CONFIG

    def get_probes_results(self, **kwargs):
        return TEST_PROBES_RESULTS

    def get_route_to(self, destination, protocol=None, **kwargs):
        assert destination == '1.2.3.4'
        return TEST_ROUTE

    def get_snmp_information(self, **kwargs):
        return TEST_SNMP_INFO

    def get_users(self, **kwargs):
        return TEST_USERS


def mock_proxy_napalm_wrap(func):
    '''
    The proper decorator checks for proxy minions. We don't care
    so just pass back to the origination function
    '''

    @wraps(func)
    def func_wrapper(*args, **kwargs):
        func.__globals__['napalm_device'] = MockNapalmDevice()
        return func(*args, **kwargs)
    return func_wrapper


import salt.utils.napalm as napalm_utils  # NOQA
napalm_utils.proxy_napalm_wrap = mock_proxy_napalm_wrap  # NOQA


def true(name):
    return True


def random_hash(source, method):
    return 12346789


def join(*files):
    return True


def get_managed_file(*args, **kwargs):
    return 'True'



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, unicode_literals, print_function
import re

# Import Salt Testing libs
from tests.support.case import ModuleCase


class KeyModuleTest(ModuleCase):
    def test_key_finger(self):
        '''
        test key.finger to ensure we receive a valid fingerprint
        '''
        out = self.run_function('key.finger')
        match = re.match("([0-9a-z]{2}:){15,}[0-9a-z]{2}$", out)
        self.assertTrue(match)

    def test_key_finger_master(self):
        '''
        test key.finger_master to ensure we receive a valid fingerprint
        '''
        out = self.run_function('key.finger_master')
        match = re.match("([0-9a-z]{2}:){15,}[0-9a-z]{2}$", out)
        self.assertTrue(match)



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, patch

# Import Salt libs
import salt.modules.config as config

DEFAULTS = {
    'test.option.all': 'value of test.option.all in DEFAULTS',
    'test.option': 'value of test.option in DEFAULTS'
}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class TestModulesConfig(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        return {
            config: {
                '__opts__': {
                    'test.option.all': 'value of test.option.all in __opts__'
                },
                '__pillar__': {
                    'test.option.all': 'value of test.option.all in __pillar__',
                    'master': {
                        'test.option.all': 'value of test.option.all in master'
                    }
                }
            }
        }

    def test_defaults_only_name(self):
        with patch.dict(config.DEFAULTS, DEFAULTS):
            opt_name = 'test.option'
            opt = config.option(opt_name)
            self.assertEqual(opt, config.DEFAULTS[opt_name])

    def test_omits(self):
        with patch.dict(config.DEFAULTS, DEFAULTS):
            opt_name = 'test.option.all'
            opt = config.option(opt_name,
                                omit_opts=False,
                                omit_master=True,
                                omit_pillar=True)

            self.assertEqual(opt, config.__opts__[opt_name])

            opt = config.option(opt_name,
                                omit_opts=True,
                                omit_master=True,
                                omit_pillar=False)

            self.assertEqual(opt, config.__pillar__[opt_name])
            opt = config.option(opt_name,
                                omit_opts=True,
                                omit_master=False,
                                omit_pillar=True)

            self.assertEqual(
                opt, config.__pillar__['master'][opt_name])
